      program hcs13_2
      
c simulation of harvest rules on a given population,
c Runs as a bootstrap
c Starts with priming with fixed F to get a population at equilibrium,
c or with initial numbers from a nin-file(numbers and variance-covariances)
c Then, runs for maxyr (normally 20) years with catches decided by a harvest rule
c Previous versions used for preliminary studies on mackerel HCRs
c summer and autumn 2007.
c This program is the general version of the former hcm program.
c
c The program was completely rewritten, october-december 2007
c and extended May  - August 2009 and throughout 2010 and 2011
c D. W. Skagen, IMR, Bergen, Norway
c
c Version hcs11_1 August 2011: Introduced multiple recruitment regimes,
c Version hcs11_2 September 2010: Introduced max deviation from the primary TAC.
c Fixed several bugs: Implementation bias now works, as well as autocorrelation on 
c observations error.
c Version 12_1, March 2012: New algorithm for obs. errors, making them more consistent over years
c                           Algorithm for stock numbers back in time in the obs. model
c                           Harvest rules based on trends
c                           Include a noisy recruitment index as decision basis
c                           Major revision of obsmodel, rule, new decision basis
c                           Cleaned up code and redundant commons
c Update 14/4-12 - bug fix line 1523, 1664, 1694 and 1870
c Extensions 18/4-12: Rule4: Logistic functions for v.
c                     Exceptions rule for filter
c Known bugs in 12_1:
c Catches in the last year are wrong
c Sometimes difference between TAC and catch that should not be there

c Version 12_3: Fixed bugs noted above, improved comparison between obs and true basis, cleaned up code.
c Version 12_5: Added two harvest rules, fixed bugs in counting spasmodic recruitment years and in VPA
c       Flexible years in output file 12
c       Adusted Y/R to mean recruitment according to recruitment distribution
c       Added surplus production to trajectories output
c Version 13_2: Added another harvest rule
c               New definition of risk in overview tables
c Version 13_2 
c       The years of catch data in the VPA in obsmodel
c       Conditions for skipping projection, and transfer of data if no projection
c       Option to take mean recruitments in project 
c       Printout of probcrash in overview tables
c       Some formatting in overview tables
c       Added Rule 8
c Last update 6 March 2013
c D.W.Skagen: Dankert Skagen Fisheries Science Consultant
c dankert@dwsk.net

c Note: This version has a hard-coded option for banking and borrowing. in lines 1460-1470
c Normally, that is neutralized by setting the bb-factor to 0 in line 1457.
c Changing it requires recompilation


      include 'hcscom13_2.i'

c Initialise random seed
      rintvar=15349668.0d0
c Define and open files for input and output
c For bio.inn there are two alternative formats, 
c indicated in the first line on the file, 
c comment out the one you do not want.
c The file names are hard coded - if you want other names you have
c to change them here and recompile
      innfilb='bio.inn'
      innfilo='opt.inn'
      innfilc='canum.inn'
      open (13,file='kladd')
c read input      
      call getpopinn
      call getoptinn
c prepare output files      
      call prepfiles
c calculate yield/recruit
       call yieldrecr
c start loop over options
      call optruns
      call closefiles
c Clear up      
      end
      
      subroutine getpopinn
      include 'hcscom13_2.i'
      character*1 intype
      character*100 streng,tempfile(100)
      open (10,file=innfilb)
c Strip away #s by reading to tempfile
      i=0
100   read(10,'(a100)',end=101) streng
      if (streng(1:1).ne.'#') then
      i=i+1
      tempfile(i)=streng
c      write(*,*) 'tempfile line',i,tempfile(i)
      endif
      goto 100
101   close (10)

c Read basic specs
      read(tempfile(1),*) iy0
      read(tempfile(2),*) lowage,iaold
      read(tempfile(3),*) iaref1,iaref2
      read(tempfile(4),*) itsbage
      read(tempfile(5),*) propf,propm
      if (iaold.gt.maxage) then
      write(*,*) 'Oldest age larger than maxage'
      write(*,*) 'Adjust maxage in hcscom12_1.i and recompile the progra
     &m'
      endif
c Recruitments
c Line with regime specs
      read(tempfile(6),*) nregimes,icodeshift
      if (nregimes.gt.1) then
       if (icodeshift.eq.1) then
        listreg(1)=1
        read(tempfile(6),*) dummy1, dummy2,(listreg(i),i=2,nregimes)
c An extra entry in listreg is filled out, to make subsequent loops easier.
        listreg(nregimes+1)=maxyr
       else if (icodeshift.eq.2) then
        read(tempfile(6),*) dummy1, dummy2,regintv,lastreg
       else
      write(*,*) 'Something wrong with specification of recruitment regi
     &mes'
      stop
       endif
      else
c case only one regime
      listreg(1)=1
      endif
      do i=0,nregimes-1
      read(tempfile(4*i+7),*) (irecrcode(i+1,j),j=1,3)
c jfunc,jdist,jtrunc
      read(tempfile(4*i+8),*) (recrparam(i+1,j),j=1,6)
c asrparam,bsrparam,sigmar,rtrunc1,rtrunc2,depens
      read(tempfile(4*i+9),*) (recrparam(i+1,j),j=7,9)
c ramplitude,rperiod,rphase
      read(tempfile(4*i+10),*) irecrcode(i+1,4),
     &(recrparam(i+1,j),j=10,12),irecrcode(i+1,5)
      enddo
c counting lines
      ilast=4*(nregimes-1)+10
c ioptspasm,rinterval,rintvcv,spasmfct,lastspasm
c SInce there are several sets of recruitment parameters, they are read to 
c arrays recrparam (for reals) and irecrcode (for integers).
c The numberisng code is:
c irecrcode:
c 1. jfunc: The code for stock recruit function (see below>)
c 2. jdist: The code for distribution of noise around the S-R function
c 3. jtrunc: The code for truncation type
c 4. ioptspasm: Code for type of spasmodic function
c 5. lastspasm: Year where counting of spasmodic recruitments start
c recrparam:
c 1. asrparam: a-parameter in SR function
c 2. bparam: b-parameter in SR function
c 3. sirmar: dispersion parameter for noise around SR function
c 4. rtrunc1: lower level for truncation
c 5. rtrunc2: upper level for truncation
c 6. depens: Depensation parameter
c 7. ramplitude: amplitude in periodic function
c 8. rperiod: period in periodic function
c 9. rphase: phase in periodic function
c 10. rinterval: Interval between spasmodic years
c 11. rintvcv: Dispersion parameter for spasmodic intervals
c 12. spasmfact: multiplier in spasmodic years
c
c key for recruitment models
c jfunc: 1: Hockey
c        2: Bev-Holt
c        3: Ricker
c jdist: 1. Normal distribution 
c        2. Lognormal distribution
c jtrunc: 1: xsi > rtrunc2 or xsi < rtrunc1: 
c         2: xm > rtrunc2 or xm < rtrunc1
c recruitment model parameters ( a and b parameters
c sigmar and truncs refer to noise multiplier
c Depens is a power term. Only for Beverton-Holt and Hockeystick
c Depens = 1 is neutral, >1 gives depensation at low ssb.
c Parameters in a periodic recruitment - ramplitude = 0 is flat
c R = R0*(1+ramplitude*cos(2pi(year-rphase)/rperiod))
c Spasmodic recruitments - sets up a list with good years (intervals lognorm random with CV), 
c spasmfct is a multiplier applied to those years. spasmfact=1.0 is neutral
c lastspasm is the last great year, relative to year 0, can be negative
c
c Read common data for weight variation
      read(tempfile(ilast+1),*) wbref
      read(tempfile(ilast+2),*) wrref
      read(tempfile(ilast+3),*) wsigma
      read(tempfile(ilast+4),*) wampl,wperiod,wphase
      read(tempfile(ilast+5),*) wycllow,wyclhigh,wyclalfa
      ilast=ilast+5
c Read type code for biological properties
      intype=tempfile(ilast+1)(1:1)
      if (intype.eq.'i'.or.intype.eq.'I') then
      intype='i'
      else if (intype.eq.'p'.or.intype.eq.'P') then
      intype='p'
      else
      write(*,*) 'Incomprehensible specification of biological input'
      write(*,*) 'file format: ',intype
      write(*,*) 'Must be i or p - correct the file and start again'
      stop
      endif
      ilast=ilast+1
c Intype = p: Read parameters for age dependent data, and generate the actual data from there.
      if (intype.eq.'p') then
c generate age-dependent data from growth model specified on the file 
      read(tempfile(ilast+1),*) rmpar
      read(tempfile(ilast+2),*) rlinf
      read(tempfile(ilast+3),*) rl0  
c guidance for growth-k: Z/1.7; Z = 2*M - 3*M
      read(tempfile(ilast+4),*) gk
      read(tempfile(ilast+5),*) condfact
      read(tempfile(ilast+6),*) slopemat
      read(tempfile(ilast+7),*) sel50
      read(tempfile(ilast+8),*) slopesel
c Density dependence parameters (common to all ages and types
      read(tempfile(ilast+9),*) factlow,facthigh,wmalfa
      ilast=ilast+9
c Finished reading for intype=p, calculate actual data:
      rkfact=1.0-exp(-gk)
c maturation at 2/3 of Linf
      rmat50=0.667*rlinf
      do 110 ia=lowage,iaold
      if (ia.eq.lowage) then
      rlact=rl0
      else
      rlact=(rlinf-rllast)*rkfact+rllast
      endif
      rllast=rlact
      rm(ia)=rmpar
      sel(ia,1)=(1.0+exp(-slopesel*(rlinf-sel50)))/
     &(1.0+exp(-slopesel*(rlact-sel50)))
      wecaspec(ia,1)=rlact**3*condfact
      westspec(ia,1)= wecaspec(ia,1)
      voksenspec(ia,1)= (1.0+exp(-slopemat*(rlinf-rmat50)))/
     &(1.0+exp(-slopemat*(rlact-rmat50)))
      wecaspec(ia,2)=factlow*wecaspec(ia,1)
      wecaspec(ia,3)=facthigh*wecaspec(ia,1)
      wecaspec(ia,4)=wmalfa
      wecaspec(ia,5)=0.0
      westspec(ia,2)=factlow*wecaspec(ia,1)
      westspec(ia,3)=facthigh*wecaspec(ia,1)
      westspec(ia,4)=wmalfa
      westspec(ia,5)=0.0
      voksenspec(ia,2)=factlow*voksenspec(ia,1)
      voksenspec(ia,3)=max(facthigh*voksenspec(ia,1),1.0)
      voksenspec(ia,4)=wmalfa
      voksenspec(ia,5)=0.0

110   continue     
 
      else
c Intype = i: read age-dependent data directly from the file
c First,       
c natural mortality
      line=ilast
      do 12 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,rm(ia)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
12    continue 

c selection at age
      do 13 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,sel(ia,1)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
13    continue 

c weca at age
      do 14 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,(wecaspec(ia,j),j=1,5)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
14    continue 

c west at age
      do 15 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,(westspec(ia,j),j=1,5)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
15    continue 
     
c proportion mature at age
      do 16 ia=lowage,iaold
      line=line+1
      read(tempfile(line),*) i,(voksenspec(ia,j),j=1,5)
      if (i.ne.ia) then
      write(*,*) 'Check ageing of biol. data input at ',i
      stop
      endif
16    continue 
      iend=(iaold-lowage+1)*5+ilast
      if (line.ne.iend) then
      write(*,*) 'WARNING!'
      write(*,*) 'The number of accepted lines on the bio.inn file'
      write(*,*) 'deviates from the number expetced'
      write(*,*) 'Expected',iend
      write(*,*) 'Found   ',line   
      write(*,*) 'Last accepted line read:',tempfile(line)
      endif
c end reading input for i or p options
      endif
c Prime weca etc. with standard values
      do ia=lowage,iaold
      weca(ia)=wecaspec(ia,1)      
      west(ia)=westspec(ia,1)    
      rmat(ia)=voksenspec(ia,1)     
      enddo 
c fill in for ages below lowage
      if (lowage.gt.0) then
      do 23 i=0,lowage-1
      rm(i)=0.0
      sel(i,1)=0.0
      weca(i)=0.0
      west(i)=0.0
      rmat(i)=0.0
23    continue
      endif  
c Transfer initial values for weights and maturities to earlier years
c for later reference
      do iy=-maxage,0
      do i=0,iaold
      trueweca(iy,i)=weca(i)
      truewest(iy,i)=west(i)
      truemat(iy,i)=rmat(i)
      enddo
      enddo
c normalize selection so that selection over (iaref1 - iaref2)=1
      avg=0.0
      do 20 ia=iaref1,iaref2
      sel(ia,2)=sel(ia,1)
      avg=avg+sel(ia,1)
20    continue
      avg=avg/float(iaref2-iaref1+1)
      do 21 ia=0,iaold
      sel(ia,1)=sel(ia,1)/avg
      sel(ia,2)=sel(ia,1)
21    continue   
      end

      
      subroutine getoptinn    
c Read management options        
      include 'hcscom13_2.i'
      character*200 streng,tempfile(100)
      open (10,file=innfilo)
c Strip away #s by reading to tempfile
      i=0
100   read(10,'(a200)',end=101) streng
      if (streng(1:1).ne.'#') then
      i=i+1
      tempfile(i)=streng
       endif
      goto 100
101   close (10)
c Initialize optrange
      do i=1,22
      optrange(i,1)=0.0
      optrange(i,2)=0.0
      optrange(i,3)=1.0
      enddo

      read(tempfile(1),*) idata
c idata: priming (1) or numbers from file (2-4).      
      if (idata.eq.1) then
c initial data from priming      
      read(tempfile(2),*) fprim
      
      else if (idata.eq.2) then
c initial data from nin.inn file
c format numbers; vsv matrix
      read(tempfile(2),*) tac0
      open (30,file='nin.inn')
c Note: The nin file is assumed to have plain numbers in the first column.
c In the matrix, the diagonal terms are covariances on the log scale,
c and the other entries are correlation coefficients.         
      do 500 ia=lowage,iaold
      read(30,*) i,rnin(ia),(vcv(ia,j),j=lowage,iaold)
      if (i.ne.ia) then
      write(*,*) 'Wrong ages on nin.inn-file'
      write(*,*) 'Stated as ',i,' Expected ',ia
      write(*,*) 'Correct the file'
      stop
      endif
500   continue    
      if (lowage.gt.0) then
      do 200 i=1,lowage
      read(30,*) pastssb(i-1)   
200   continue
      endif
      close (30)  

c check the vcv for symmetry
      do 501 i=lowage,iaold
      do 502 j=i,iaold
      if (vcv(i,j).ne.vcv(j,i)) then
      write(*,*) 'Correlation matrix in ninfile is not symmetric'
      write(*,*) i,j,vcv(i,j) 
      write(*,*) 'is different from '
      write(*,*) j,i,vcv(j,i)
      write(*,*) 'Correct the file'
      stop
      endif
502   continue
501   continue  
c convert correlations to covariances     
      do 110 i=lowage,iaold
      do 110 j=i+1,iaold
      vcv(i,j)=vcv(i,j)*vcv(i,i)*vcv(j,j)
      vcv(j,i)=vcv(i,j)
110   continue

                    
      else if (idata.eq.3) then
c if input form file with numbers and variances & year facor
      read(tempfile(2),*) tac0
      open (30,file='nin.num')
c format: age, number at age, sigma at age
      do 250 ia=lowage,iaold
      read (30,*) i,r1,r2
      if (i.ne.ia) then
      write(*,*) 'Wrong ages in the nin.num file'
      write(*,*) 'Stated as ',i,' Expected ',ia
      write(*,*) 'Correct the file and start again'
      stop
      endif
      rnin(ia)=r1
      cvnin(ia)=r2
c      write(*,*) ia,r1,r2
250   continue
             
      if (lowage.gt.0) then
      do 201 i=1,lowage
      read(30,*) pastssb(i-1)   
201   continue
      endif        
      close (30)  
      
      else if (idata.eq.4) then
c if input form file with numbers only, linked to using the 
c obs-model for generating the starting numbers
      read(tempfile(2),*) tac0
      open (30,file='nin.obs')
c format: age, number at age
      do 251 ia=lowage,iaold
      read (30,*) i,r1
      if (i.ne.ia) then
      write(*,*) 'Wrong ages in the nin.obs file'
      write(*,*) 'Stated as ',i,' Expected ',ia
      write(*,*) 'Correct the file and start again'
      stop
      endif
      rnin(ia)=r1
251   continue
      if (lowage.gt.0) then
      do 202 i=1,lowage
      read(30,*) pastssb(i-1)   
202   continue
      endif
      close (30)  
      
      else
      write(*,*) 'Invalid option for year 0: ',idata
      stop
      endif
c read the rest of the options file
          
c fmax: maximum F that can be achieved. Protection of shortage of fish.
      read(tempfile(3),*) fmax
c lastassn: Time gap between assessment year and last year with 
c Normally negative, i.e. start projection through intermediate
c year in year iy+lastassn
      read(tempfile(4),*) lastassn
c Interval between decision years
      read(tempfile(5),*) idur
c Reference Blim
      read(tempfile(6),*) blim
c Youngest age in TSB
      read(tempfile(7),*) ibplusage
c Read specification of rule basis
      read(tempfile(8),*) nbasis
c iobsstart is the earliest year that is used in the asssessment table, typically negative.
      iobsstart=lastassn
      iproj=0
      line=9
      do i=1,nbasis
      read(tempfile(line),*) ii,(ibastable(i,j),j=1,4)
      if (ii.ne.i) write(*,*) 'Warning: decision basis confused'
      if (ibastable(i,2).lt.iobsstart) iobsstart=ibastable(i,2)
c      write(*,*) 'input',i,ibastable(i,2),iobsstart
      if (ibastable(i,3).gt.iproj) iproj=ibastable(i,3)
      if (ibastable(i,3).gt.0.and.ibastable(i,1).eq.3) then
      write(*,*) 'You ask for recruitment indicator for years'
      write(*,*) 'after the current year, which are undefined'
      write(*,*) 'Check the opt.inn file'
      stop
      endif 
      line=line+1
      enddo
c Read rule and rule measure
      read(tempfile(line),*) irule,ibasrule
      line=line+1
      if (irule.eq.1) then
      npar=5
      else if (irule.eq.2) then
      npar=6
      else if (irule.eq.3) then
      npar=6
      else if (irule.eq.4) then
      npar=6
      else if (irule.eq.5) then
      npar=7
      else if (irule.eq.6) then
      npar=8
      else if (irule.eq.7) then
      npar=5
      else if (irule.eq.8) then
      npar=7
      else
      write(*,*) 'Asked for non-existing rule:', irule
      stop
      endif
c read rule parameters into optrange
c Note: optrange is dimensioned for up to 10 rule parameters, only those used are filled in
      do i=1,npar
c      write (*,*) i,line,tempfile(line)
      read (tempfile(line),*) (optrange(i,j),j=1,3)
      line=line+1
      enddo
c Read constraint rules
c The 3 rules must be on the file, but can be given values that are 
c ignored
      do i=1,3
      read(tempfile(line),*) icrule

      if (icrule.eq.1) then
      line=line+1
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) (optrange(11,j),j=1,3)
      line=line+1
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) (optrange(12,j),j=1,3)
      line=line+1
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) (icbastable(1,j),j=1,4)
      line=line+1      

      else if (icrule.eq.2) then
      read(tempfile(line),*) idummy,ialwonly
      line=line+1
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) (optrange(13,j),j=1,3)
      line=line+1
      if (ialwonly.eq.2) then
      read(tempfile(line),*) (optrange(14,j),j=1,3)
      line=line+1
c       write (*,*) line,tempfile(line)
      read(tempfile(line),*) (icbastable(2,j),j=1,4)
      line=line+1      
      endif
      if (icbastable(2,2).lt.iobsstart) iobsstart=icbastable(2,2)
      if (icbastable(2,3).gt.iproj) iproj=icbastable(2,3)
      if (icbastable(2,3).gt.0.and.icbastable(2,1).eq.3) then
      write(*,*) 'You ask for recruitment indicator for years'
      write(*,*) 'after the current year, which are undefined'
      write(*,*) 'Check the opt.inn file'
      stop
      endif 

      else if (icrule.eq.3) then
      line=line+1
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) (optrange(15,j),j=1,3)
      line=line+1
c      write (*,*) line,tempfile(line)
c      write(*,*) 'min',(optrange(15,j),j=1,3)
      read(tempfile(line),*) (optrange(16,j),j=1,3) 
c      write(*,*) 'max',(optrange(16,j),j=1,3)
      line=line+1   
      else
      write(*,*) 'Asked for non-existing constraint option', icrule 
      endif
      enddo
c iabrgrad: Change throughout multiple years: 1: Abrupt, 2: Gradual
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) iabrgrad
      line=line+1
c Gradual transition to target:Number of years to reach target (1 is from the outset)
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) (optrange(17,j),j=1,3) 
      line=line+1
c Obs model noise
      do i=18,23
c      write (*,*) line,tempfile(line)
      read(tempfile(line),*) (optrange(i,j),j=1,3)
      line=line+1
      enddo
c Code for screened options parameters (lower, upper, interval), kept in optrange:
c      1-10. Parameters in rule function
c      11-16: Parameters in Constraints
c              11(-12): Constraint 1
c              13-14: Constraint 2
c              15-16: Constraint 3   
c      17: Gradual transition
c      18. Bias obs model
c      19. Bias implementation models
c      20. CV year factor obs model
c      21. CV year factor implementation model
c      22. CV recruitment survey
c      23. CV biomass survey

      do ia=lowage,iaold
c      write (*,*) line,tempfile(line)
      read (tempfile(line),*) i,r1,r2,r3
      line=line+1
      if (i.ne.ia) then
      write(*,*) 'Wrong ages for cv-values in the opt file'
      write(*,*) 'Stated as ',i,' Expected ',ia
      write(*,*) 'Correct the file and start again'
      stop
      endif
      cvobs(ia,1)=r1
      cvobs(ia,2)=r2
      cvimp(ia)=r3
      enddo
c read coefficients for autoregressive obs. error
c Initialize first. Dimension is maxage, more is never needed
      do j=0,maxage
      acfact(j)=0.0
      enddo
      read(tempfile(line),*) nac
      read(tempfile(line),*) ndummy,(acfact(j),j=0,nac-1)
      line=line+1
      read(tempfile(line),*) nrecryears
      if (iobsstart.gt.-nrecryears) iobsstart=-nrecryears

      line=line+1
c Read list for printout options
c This list, with reference to file numbers, declares if
c this kind of output shall be made. The files are always opened,
c to avoid accidental writing to files not open, but will
c not be written to if the rules are followed.      
      do i=11,26
      ioutopt(i)=0  
      if (i.ne.13) then
      read(tempfile(line),*) ii,j
      if (i.eq.12.and.j.eq.1) 
     &read(tempfile(line),*) idummy,idummy,intvpr1,intvpr2
      line=line+1
      ioutopt(ii)=j
      endif
      enddo
c If needed, read canum file
      if (iobsstart.lt.0) then
      do i=0,maxage
      do j=-maxage,maxyr+mbasis
      cobs(i,j)=0.0
      enddo
      enddo
      open (10,file=innfilc)
532   read(10,'(a200)',end=520) streng
      read(streng,*) i
      iy=i-iy0
c      write(*,*) iy,streng,lowage,iaold
      if (iy.ge.-maxage) 
     &read(streng,*) idummy,(cobs(ia,iy),ia=lowage,iaold)
c      write(*,*) 'cobs',iy,(cobs(ia,iy),ia=lowage,iaold)
      goto 532
520   close(10)
c       write(*,*) 'options read'
      endif

c decide need for projection
c iproj is determined already by the need for basis
c add one year if propf or propm >0
      if (propm.gt.0.0.or.propf.gt.0.0) iproj=iproj+1
      
      end
      
      subroutine prepfiles
c opens files and writes headers for files that collect results from multiple runs
      include 'hcscom13_2.i'
      character*10 streng
      streng='Years___  '
      if (ioutopt(12).eq.1) then
      write(streng(6:7),'(i2)') intvpr1
      write(streng(9:10),'(i2)') intvpr2
      endif
      
      if (ioutopt(11).eq.1) open (11,file='Results')
      if (ioutopt(12).eq.1) open (12,file=streng)
      if (ioutopt(13).eq.1) open (13,file='kladd')
      if (ioutopt(14).eq.1) open (14,file='First_5')
      if (ioutopt(15).eq.1) open (15,file='Traj')
      if (ioutopt(16).eq.1) open (16,file='S0_distr')
      if (ioutopt(17).eq.1) open (17,file='F_distr')
      if (ioutopt(18).eq.1) open (18,file='IAV_distr')
      if (ioutopt(19).eq.1) open (19,file='compbas')
      if (ioutopt(20).eq.1) open (20,file='SRpairs')
      if (ioutopt(21).eq.1) open (21,file='All_years')
      if (ioutopt(22).eq.1) open (22,file='Yieldrecr')
      if (ioutopt(23).eq.1) open (23,file='Catch_agedist')
      if (ioutopt(24).eq.1) open (24,file='SSB_agedist')
      if (ioutopt(25).eq.1) open (25,file='Wema')
      if (ioutopt(26).eq.1) open (26,file='Comprecr')

c write headers for files collecting results from all runs      
  
      if (irule.eq.1) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Filter Except_F Perc
     &ent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_impl C
     &V_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     &SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(14).eq.1) 
     &write(14,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Filter Except_F Perc
     &ent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_impl C
     &V_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     &SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(21).eq.1) 
     &write(21,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Filter Except_F Perc
     &ent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_impl C
     &V_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     &SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
      else if (irule.eq.2) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'B0 Btrig1 Btrig2 Target Level1 Alfa2 Filter Except_F
     &Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_im
     &pl CV_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SS
     &B10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash
     &'
      if (ioutopt(14).eq.1) 
     &write(14,*) 'B0 Btrig1 Btrig2 Target Level1 Alfa2 Filter Except_F
     &Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_im
     &pl CV_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SS
     &B10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash
     &'
      if (ioutopt(21).eq.1) 
     &write(21,*) 'B0 Btrig1 Btrig2 Target Level1 Alfa2 Filter Except_F
     &Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_im
     &pl CV_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SS
     &B10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash
     &'
      else if (irule.eq.3) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Gain Filter Execpt_F
     &er Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr
     &_impl  CV_recr CV_Biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smea
     &n SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig C
     &rash'
       if (ioutopt(14).eq.1) 
     &write(14,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Gain Filter Execpt_F
     &er Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr
     &_impl  CV_recr CV_Biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smea
     &n SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig C
     &rash'
       if (ioutopt(21).eq.1) 
     &write(21,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Gain Filter Execpt_F
     &er Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr
     &_impl  CV_recr CV_Biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smea
     &n SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig C
     &rash'
      else if (irule.eq.4) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'Btrig1 Btrig2 Target Slope1 Slope2 Vmax Filter Except
     &_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr
     &_impl  CV_recr CV_Biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smea
     &n SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig C
     &rash'
       if (ioutopt(14).eq.1) 
     &write(14,*) 'Btrig1 Btrig2 Target Slope1 Slope2 Vmax Filter Except
     &_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr
     &_impl  CV_recr CV_Biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smea
     &n SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig C
     &rash'
       if (ioutopt(21).eq.1) 
     &write(21,*) 'Btrig1 Btrig2 Target Slope1 Slope2 Vmax Filter Except
     &_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr
     &_impl  CV_recr CV_Biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smea
     &n SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig C
     &rash'
 
      else if (irule.eq.5) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'Btrig1 Btrig2 RefRec Alfa1 Alfa2 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs 
     &CVyr_impl  CV_recr Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     & SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(14).eq.1) 
     &write(14,*) 'Btrig1 Btrig2 RefRec Alfa1 Alfa2 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs 
     &CVyr_impl  CV_recr Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     & SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(21).eq.1) 
     &write(21,*) 'Btrig1 Btrig2 RefRec Alfa1 Alfa2 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs 
     &CVyr_impl  CV_recr Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     & SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'

      else if (irule.eq.6) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'B0 Btrig1 Btrig2 RefRec Alfa2 V0 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs
     &CVyr_impl  CV_recr Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10
     & SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(14).eq.1) 
     &write(14,*) 'B0 Btrig1 Btrig2 RefRec Alfa2 V0 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs
     &CVyr_impl  CV_recr Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10
     & SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(21).eq.1) 
     &write(21,*) 'B0 Btrig1 Btrig2 RefRec Alfa2 V0 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs
     &CVyr_impl  CV_recr Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10
     & SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
      else if (irule.eq.7) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Filter Except_F Perc
     &ent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_impl C
     &V_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     &SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(14).eq.1) 
     &write(14,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Filter Except_F Perc
     &ent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_impl C
     &V_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     &SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
       if (ioutopt(21).eq.1) 
     &write(21,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Filter Except_F Perc
     &ent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_impl C
     &V_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 Smean SSB10 
     &SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtrig Crash'
      else if (irule.eq.8) then
      if (ioutopt(12).eq.1) 
     &write(12,*) 'Btrig1 Btrig2 Vstd   Alfa1 Alfa2 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs 
     &CVyr_impl  CV_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 
     &Smean SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtr
     &ig Crash'
       if (ioutopt(14).eq.1) 
     &write(14,*) 'Btrig1 Btrig2 Vstd   Alfa1 Alfa2 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs 
     &CVyr_impl  CV_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 
     &Smean SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtr
     &ig Crash'
       if (ioutopt(21).eq.1) 
     &write(21,*) 'Btrig1 Btrig2 Vstd   Alfa1 Alfa2 Vlow Vhigh Filter Ex
     &cept_F Percent Except_P MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs 
     &CVyr_impl  CV_recr CV_biosurv Cmean C10 C50 C90 Fmean F10 F50 F90 
     &Smean SSB10 SSB50 SSB90 Tmean TSB10 TSB50 TSB90 IAV Risklim Probtr
     &ig Crash'

      endif
      end

 
 
      
      subroutine closefiles
      include 'hcscom13_2.i'
      if (ioutopt(11).eq.1) close (11)
      if (ioutopt(12).eq.1) close (12)
      if (ioutopt(13).eq.1) close (13)
      if (ioutopt(14).eq.1) close (14)
      if (ioutopt(15).eq.1) close (15)
      if (ioutopt(16).eq.1) close (16)
      if (ioutopt(17).eq.1) close (17)
      if (ioutopt(18).eq.1) close (18)
      if (ioutopt(19).eq.1) close (19)
      if (ioutopt(20).eq.1) close (20)
      if (ioutopt(21).eq.1) close (21)
      if (ioutopt(25).eq.1) close (25)
      if (ioutopt(26).eq.1) close (26)
c files 22,23 and 24 are closed by the yieldrecr routine
      end
      
 
      subroutine optruns
c Controls the screeing over options, calls bootruns
c for each new set of options
      include 'hcscom13_2.i'
      integer nsteps(23)
c check that no optrange is the wrong way around
      do i=1,23
      if (optrange(i,2).lt.optrange(i,1)) then
      write(*,*) 'Mis-specified options range, option',i
      write(*,*) (optrange(i,j),j=1,3)
      stop
      endif
      enddo
c convert optranges to integers
      do i=1,23
      if (optrange(i,3).gt.0.0) then
      nsteps(i)=int((optrange(i,2)*1.001-optrange(i,1))/optrange(i,3))
      else
      nsteps(i)=0
      endif
      enddo
c Code for screened options parameters (lower, upper, interval), kept in optrange:
c      1-10. Parameters in rule function; Details in each rule function
c
c      11-16: Parameters in Constraints
c              11(-12): Constraint 1 12 is not used at present
c              13-14: Constraint 2
c              15-16: Constraint 3   
c      17: Gradual transition
c      18. Bias obs model
c      19. Bias implementation models
c      20. CV year factor obs model
c      21. CV year factor implementation model
c      22. CV recruitment survey
c      23. CV biomass survey

      do 905 i1=0,nsteps(1)
      h1=optrange(1,1)+float(i1)*optrange(1,3)
      do 905 i2=0,nsteps(2)
      h2=optrange(2,1)+float(i2)*optrange(2,3)
      do 905 i3=0,nsteps(3)
      h3=optrange(3,1)+float(i3)*optrange(3,3)
      do 905 i4=0,nsteps(4)
      h4=optrange(4,1)+float(i4)*optrange(4,3)
      do 905 i5=0,nsteps(5)
      h5=optrange(5,1)+float(i5)*optrange(5,3)
      do 905 i6=0,nsteps(6)
      h6=optrange(6,1)+float(i6)*optrange(6,3)
      do 905 i7=0,nsteps(7)
      h7=optrange(7,1)+float(i7)*optrange(7,3)
      do 905 i8=0,nsteps(8)
      h8=optrange(8,1)+float(i8)*optrange(8,3)
      do 905 i9=0,nsteps(9)
      h9=optrange(9,1)+float(i9)*optrange(9,3)
      do 905 i10=0,nsteps(10)
      h10=optrange(10,1)+float(i10)*optrange(10,3)
      do 905 i11=0,nsteps(11)
      h11=optrange(11,1)+float(i11)*optrange(11,3)
      do 905 i12=0,nsteps(12)
      h12=optrange(12,1)+float(i12)*optrange(12,3)
      do 905 i13=0,nsteps(13)
      h13=optrange(13,1)+float(i13)*optrange(13,3)
      do 905 i14=0,nsteps(14)
      h14=optrange(14,1)+float(i14)*optrange(14,3)
      do 905 i15=0,nsteps(15)
      h15=optrange(15,1)+float(i15)*optrange(15,3)
      do 905 i16=0,nsteps(16)
      h16=optrange(16,1)+float(i16)*optrange(16,3)
      do 905 i17=0,nsteps(17)
      h17=optrange(17,1)+float(i17)*optrange(17,3)
      do 905 i18=0,nsteps(18)
      h18=optrange(18,1)+float(i18)*optrange(18,3)
      do 905 i19=0,nsteps(19)
      h19=optrange(19,1)+float(i19)*optrange(19,3)
      do 905 i20=0,nsteps(20)
      h20=optrange(20,1)+float(i20)*optrange(20,3)
      do 905 i21=0,nsteps(21)
      h21=optrange(21,1)+float(i21)*optrange(21,3)
      do 905 i22=0,nsteps(22)
      h22=optrange(22,1)+float(i22)*optrange(22,3)
      do 905 i23=0,nsteps(23)
      h23=optrange(23,1)+float(i23)*optrange(23,3)

      hcrparam(1)=h1
      hcrparam(2)=h2
      hcrparam(3)=h3
      hcrparam(4)=h4
      hcrparam(5)=h5
      hcrparam(6)=h6
      hcrparam(7)=h7
      hcrparam(8)=h8
      hcrparam(9)=h9
      hcrparam(10)=h10
      hcrparam(11)=h11
      hcrparam(12)=h12
      hcrparam(13)=h13
      hcrparam(14)=h14
      hcrparam(15)=h15
      hcrparam(16)=h16
      hcrparam(17)=h17
      hcrparam(18)=h18
      hcrparam(19)=h19
      hcrparam(20)=h20
      hcrparam(21)=h21
      hcrparam(22)=h22
      hcrparam(23)=h23
c  For each option, initialise arrays for assembling results 
      do 331 iy=0,maxyr
      do 333 iter=1,niter
      truessb(iy,iter)=-1.0
      truecatch(iy,iter)=-1.0
      tac(iy,iter)=-1.0
      truef(iy,iter)=-1.0      
      obsssb(iy,iter)=-1.0
      obsf(iy,iter)=-1.0
      iflag(iy,iter)=0
333   continue      
331   continue

c write headers for output of individual runs
      if (ioutopt(11).eq.1) then 
      write(11,*) 'Options for this run:'
      call header(11)
      endif
      do 790 ifil=15,20
      if (ioutopt(ifil).eq.1) call header(ifil)
790   continue          
      do 791 ifil=25,26
      if (ioutopt(ifil).eq.1) call header(ifil)
791   continue    
      write(*,*) (hcrparam(ii),ii=1,23)  
      cvrecrsurv=hcrparam(22)
      cvbiosurv=hcrparam(23)
c      write(*,*) 'call bootruns'

      call bootruns
      call bootprint
905   continue
      end

      
      
      subroutine header(ifil)    
c Headers in files with results from individual runs  
      include 'hcscom13_2.i'
      integer ifil


      if (irule.eq.1) then
      write(ifil,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Filter Trig_filt P
     &ercent Trig_perc MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVyr_im
     &pl CV_recr CV_biosurv'
      write(ifil,101) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)
      else if (irule.eq.2) then
      write(ifil,*) 'B0 Btrig1 Btrig2 Target Level1 Alfa2 Filter Trig_fi
     &lt Percent Trig_perc MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CVy
     &r_impl CV_recr CV_biosurv'
      write(ifil,102) (hcrparam(i),i=1,6),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)
      else if (irule.eq.3) then     
      write(ifil,*) 'Btrig1 Btrig2 Target Alfa1 Alfa2 Gain Filter Trig_f
     &ilt Percent Trig_perc MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs CV
     &yr_impl CV_recr CV_biosurv'
      write(ifil,102) (hcrparam(i),i=1,6),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)
      else if (irule.eq.4) then  
      write(ifil,*) 'Btrig1 Btrig2 Target Slope1 Slope2 Vmax Filter Trig
     &_filt Percent Trig_perc MinTAC MaxTAC Bias_obs Bias_impl CVyr_obs 
     &CVyr_impl CV_recr CV_biosurv'
      write(ifil,102) (hcrparam(i),i=1,6),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)
      else if (irule.eq.5) then  
      write(ifil,*) 'Btrig1 Btrig2 RefRec Alfa1 Alfa2 Vlow Vhigh Filter 
     &Trig_filt Percent Trig_perc MinTAC MaxTAC Bias_obs Bias_impl CVyr_
     &obs CVyr_impl CV_recr CV_biosurv'
      write(ifil,103) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)
      else if (irule.eq.6) then  
      write(ifil,*) 'B0 Btrig1 Btrig2 RefRec Alfa2 V0 Vlow Vhigh Filter 
     &Trig_filt Percent Trig_perc MinTAC MaxTAC Bias_obs Bias_impl CVyr_
     &obs CVyr_impl CV_recr CV_biosurv'
      write(ifil,104) (hcrparam(i),i=1,8),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)
      else if (irule.eq.7) then
       write(ifil,101) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)
      else if (irule.eq.8) then  
      write(ifil,*) 'Btrig1 Btrig2 Vstd Alfa1 Alfa2 Vlow Vhigh Filter 
     &Trig_filt Percent Trig_perc MinTAC MaxTAC Bias_obs Bias_impl CVyr_
     &obs CVyr_impl CV_recr CV_biosurv'
      write(ifil,103) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23)

      endif
101   format(2f10.1,f10.3,3f6.3,f10.1,f6.2,3f10.1,6f6.3)
102   format(3f10.1,2f10.3,2f6.3,f10.1,f6.2,3f10.1,6f6.3)
103   format(3f10.1,2f6.3,2f10.3,f6.3,f10.1,f6.1,3f10.1,4f6.3)
104   format(4f10.1,2f6.3,2f10.3,f6.3,f10.1,f6.1,f10.1,6f6.3)
      write(ifil,*)   


      if (ifil.eq.17) write(17,*) 'Iter  Year Decid   Real'
      if (ifil.eq.18) write(18,*) 'Iter   Year     IAV' 
      if (ifil.eq.19) write(19,*) 'Year True   Obs' 
      if (ifil.eq.20) write(20,*) 'Iter  Year     SSB      Recr'
      if (ifil.eq.26) write(26,*) 'Iter  Year     TrueR      ObsR'

      end
      
      subroutine bootruns      
      include 'hcscom13_2.i'
c Local variables.

      real truen(0:maxage,0:maxyr),
     &rn(0:maxage),rn1(0:maxage),tactemp(0:maxyrp),
     &projn(0:maxage,0:maxyrp),geodist(0:maxyr)
      real canum(0:maxage)
c start sequence of random numbers in the same place for each run      
      rintvar=15349668.0d0
      
c loop over niter iterations 
      do 999 iter=1,niter

c Set up regime shifts      
      if (nregimes.le.1) then
c case no regimeshifts
      do i=0,maxyr
      nregact(i)=1
      enddo
      


      else 
c nregimes >1
c Set up sequence of regimes for recruitment
c If icodeshift=1, listreg is ready from input
c If icodeshift=2, listreg is set up here      
       if (icodeshift.eq.2) then   
        call creategeo(geodist,regintv,maxyr)
        listreg(1)=1
        ishift=lastreg
        nn=1
c Find next shift
288     x=rnx(rintvar)
        ii=1
287     ii=ii+1
        if (geodist(ii).ge.x) goto 289
        if (ii.lt.maxyr) goto 287
289     ishift=ishift+ii
        if (ishift.gt.0.and.ishift.le.maxyr.and.nn.le.maxyr) then
         nn=nn+1
         listreg(nn)=ishift
        endif
        if (ishift.lt.maxyr) goto 288
        listreg(nn+1)=maxyr

        else 
c icodeshift not 2       
         if (icodeshift.ne.1) then
        
         write(*,*) 'Incomprehensible code for regime shift', icodeshift
         stop
        endif 
       endif
      endif
c Known shift years are now in listreg (fixed or derived from flex model)
c
c Set up nregact: Regime by year.
c Always start with regime 1, so prime nregact with that
      do i=0,maxyr
      nregact(i)=1 
      enddo  
      if (nregimes.gt.1) then
 
      j=1
c for each regime block, define the years
c The last entry on listreg shall be maxyr
501   j=j+1
      iyfirst=listreg(j)
      iylast=listreg(j+1)-1
      if (listreg(j+1).eq.maxyr) iylast=maxyr
c Then, find the regime number k for those years
      if (icodeshift.eq.1) then
c Fixed sequence: Consecutive regimes
      k=j
      else if (icodeshift.eq.2) then
c Random regimes
c Find which regime k in this interval, each is equally likely
      x=rnx(rintvar)
      x=x*float(nregimes)
      k=int(x)+1      
c put in place
      endif
      do iy=iyfirst,iylast
      nregact(iy)=k
      enddo
c end loop over regimes
      if (iylast.lt.maxyr) goto 501
      endif



c set up the sequence of spasmodic recruitment successes.  
c Get parameters for the regime in question
      do i=-maxage,maxyr
      ispasm(i)=0
      enddo
c set up spasmodic years for each regime
      do k=1,nregimes
      rinterval=recrparam(k,10)
      rintvcv=recrparam(k,11)
      spasmfct=recrparam(k,12)
      ioptspasm=irecrcode(k,4)
      if (k.eq.1) then
       iystart=0
      else
      iystart=listreg(k)
      endif
       if (k.eq.nregimes) then
      iyend=maxyr
      else
      iyend=listreg(k+1)
      endif
      lastspasm=iystart+irecrcode(k,5)
c       write(*,*) 'iystart',k,lastspasm
      if (rinterval.gt.1.0.and.spasmfct.gt.1.0) then
c only if the interval and spasmodic factor is greater than 1, 
c if not, no such effects are assumed


      if (ioptspasm.eq.1) then
c case spasms with random intervals
      rilast=float(lastspasm)
199   x=rnx(rintvar)
      xsi=snrn(x)
      ri=rinterval*exp(rintvcv*xsi)+rilast
      j=int(ri+0.5)
      if (j.le.iyend.and.j.ge.iystart) then
c Note: The spasmodic years start in lastspasm
c Years prior to year 0 only matter when priming the stock.
c Else, earlier years are only used to propagate the sequence. 
      ispasm(j)=1
c      write(*,*) j
      rilast=ri
      goto 199
      endif

      else if (ioptspasm.eq.2) then

       pr=1.0/rinterval
       ilast=lastspasm
c      if (iter.eq.1) write(*,*) lastspasm
297    x=rnx(rintvar)
       ii=0
       px=0.0
299    ii=ii+1
       flii=float(ii)-1.0
       px=px+pr*exp(flii*log(1.0-pr))
       if (px.ge.x) then
        goto 298
       else 
        if (ii.lt.maxyr) then
         goto 299
        else
         goto 298
        endif
       endif
298   j=ii+ilast
      ilast=j   
       if (j.le.iyend.and.j.ge.iystart) then
c Note: The spasmodic years start in lastspasm
c Years prior to year 0 only matter when priming the stock.
c Else, earlier years are only used to propagate the sequence. 
        ispasm(j)=1
c        write(*,*) 'spasm',j
        goto 297
       endif

       else if (ioptspasm.eq.3) then
c Flat distribution between rinterval*(1-cv,1+cv)
       ilast=lastspasm
398    x=rnx(rintvar)
       x=rinterval*(x*rintvcv+1.0-rintvcv/2.0)
       ii=int(x+0.5)+ilast
        if (ii.lt.maxyr) then
        if (ii.ge.-maxage) ispasm(ii)=1
        ilast=ii
c        write(*,*) x,int(x+0.5),ii,ilast
        
         goto 398
        endif
       endif


c end if spasmodic recrutimets
      endif
c end regimes loop
      enddo

c Initialise local arrays      
      do 10 ia=0,iaold
      rn(ia)=-1.0
      rn1(ia)=-1.0
      weca(ia)=trueweca(0,ia)
      west(ia)=truewest(0,ia)
      rmat(ia)=truemat(0,ia)
      do iy=0,maxyr
      truen(ia,iy)=-1.0
      projn(ia,iy)=-1.0
      enddo
      do iy=-maxage,maxyr
      wyclfct(iy)=1.0
      biosurv(iy,1)=-1.0
      biosurv(iy,2)=-1.0
      enddo
10    continue     

c Build initial xsiobs and cobs
      do iy=-iaold,0
      x=rnx(rintvar)
      xsi=snrn(x)*yfctobs
      btemp=(1.0+biasobs)*exp(xsi-yfctobs**2/2.0)
c btemp is the common random multiplier
c By age
      do ia=0,iaold
      if (ia.lt.lowage) then
      xsiobs(ia,iy)=1.0
      else
      if (cvobs(ia,1).gt.0.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cvobs(ia,1)
      else
      xsi=0.0
      endif
      xsiobs(ia,iy)=btemp*exp(xsi-cvobs(ia,1)**2/2.0)
      endif
      enddo
      enddo
c get initial data      
      if (idata.eq.1) then
      call prime(rn)
      else if (idata.eq.2.or.idata.eq.3) then
      call getactn(rn)
      else if (idata.eq.4) then
c option 4 (Input N with noise according to obsmodel
      call obsmodel(rnin,0,0)
      do ia=0,maxage
      rn(ia)=rnobs(ia,0,0)
      rnproj(ia,0)=rn(ia)
      enddo
      else
      write(*,*) 'Incomprehensible code for initial numbers 0 ',idata
      stop
      endif

      if (idata.gt.1.and.lowage.gt.0) then
c get previous recruits to fill in the ages before recruiting age
c not needed when priming
      do 220 i=0,lowage-1
      s=pastssb(lowage-i-1)
      ps=sigmar
      iy=-i
      call rcrparam(iy,1)
      call recr(r0,s,ps)
      rn(i)=r0
c Use the yearclass factor for weights for the new year class
      wyclfct(iy)=wyclfct0
220   continue
      endif

c Keep rn as true for year 0
      do 11 ia=0,iaold      
11    truen(ia,0)=rn(ia)
      truerecr(0,iter)=rn(0)

c Year 0
      iy=0
      if (idata.eq.1) then
c Fint TAC and F for year 0, in y0 and f0, applying true n in rn
c if priming, priming F is assumed for year 0
      f0=fprim
      call yield(rn,y0,f0,1)
      else if (idata.gt.1) then
c If nin, TAC constraint is assumed in year 0 
      y0=tac0     
      call findf(rn,y0,f0,jflg,1)
      else
      stop
      endif   
 
c put true values in place  
      call calcssb(rn,s,f0,1)
      truessb(0,iter)=s
      call calctsb(rn,tsb)
      truetsb(0,iter)=tsb
      truef(0,iter)=f0
      truecatch(0,iter)=y0
      tac(0,iter)=y0


c Implement the TAC, get true catch, F SSB and project true Ns      
      call implement(tac(0,iter),truecatch(0,iter),rn,
     &truef(0,iter),jflg,canum)   
c NOTE: The true catch if the TAC is too large for the stock, is the 
c realized catch at Fmax 
c Get observed catches by putting errors to the implemented catches
      call getobscanum(canum,0)

c get distorted data for year 0 
      call obsmodel(rn,0,1)
   
c find F and SSB according to assumed exploitation and distorted Ns      
      if (idata.ge.2) then
      call findf(rn1,y0,fshit,jflg1,1)
      obsf(0,iter)=fshit
      call calcssb(rn1,sshit,fshit,1)
      else
      obsf(0,iter)=f0
      call calcssb(rn1,sshit,f0,1)
      endif
      obsssb(0,iter)=sshit  
c project true stock into next year  
      call rcrparam(iy,1)
      call updatewema(iy,rn)
      call nextyear(rn,f0,s,sigmar,1)
      call obsrecruit(iy,rn(0),cvrecrsurv)
      call obssurvey(iy,rn,f0,cvbiosurv,1)
      do 12 ia=0,iaold
12    truen(ia,1)=rn(ia)
      truerecr(1,iter)=rn(0)
c Use the yearclass factor for weights for the new year class
      wyclfct(iy+1)=wyclfct0
c build up ruletarget: Target values that may deviate from the final target
c in hcrparam(2)
      tstep=(obsf(0,iter)-hcrparam(2))/hcrparam(3)  
      iydone=min(int(hcrparam(17)+0.00001),maxyr)
      ruletarget(0)=obsf(0,iter)
      do i=1,iydone
      ruletarget(i)=ruletarget(i-1)-tstep
      enddo
      if (iydone.lt.maxyr) then
      do i=iydone+1,maxyr
      ruletarget(i)=hcrparam(2)
      enddo
      endif


c years 1-- maxyr
      do 20 iy=1,maxyr
     
c Get get true ns (at start of year iy+lastassn) in rn and distorted Ns in rn1
c Also,get the 7+ percentage      
      summ1=0.0
      summ2=0.0
      do 102 ia=0,iaold
      rn(ia) = truen(ia,iy)
      if (ia.ge.itsbage) summ1=summ1+rn(ia)
102   if (ia.ge.7) summ2=summ2+rn(ia)
      rn7(iy,iter)=summ2/summ1*100.0
      call obsmodel(rn,iy,1)

c call hcr if there is no tac yet for iy   
c Note: Internally, hcr uses the weights from last assessment year.
c The weights are reset to current year at the end of hcr.
      if (tac(iy,iter).lt.0.0) call hcr(tactemp,iy)
      do i=iy,iy+idur-1
      tac(i,iter)=tactemp(i)
      enddo
c ******************************************************      
c bank-borrow factor specific to the Blue whiting study
        bb=0.0
c bb=0.0 is neutral        
c Set the tac with bb
      tacbb=(1.0-bb)*tac(iy,iter)+bb*tac(iy-1,iter)
      call implement(tacbb,truecatch(iy,iter),rn,
     &truef(iy,iter),jflg,canum)   
c The true catch, canum and F correspond to the catch that is actually taken, including
c also implementation error in sub implementation     
c *******************************************************        

c Implement the TAC, get true catch, F SSB and project true Ns      
c      call implement(tac(iy,iter),truecatch(iy,iter),rn,
c     &truef(iy,iter),jflg,canum)   
c NOTE: The true catch if the TAC is too large for the stock, is the 
c realized catch at Fmax 
c Get observed catches by putting errors to the implemented catches
      call getobscanum(canum,iy)

c get current true SSB (needed for recruitment)       
      call calcssb(rn,s,truef(iy,iter),2)
      truessb(iy,iter)=s
      call calctsb(rn,tsb)
      truetsb(iy,iter)=tsb

c iflag, i.e. crash is either if the TAC cannot be taken (jflg=1) and/or 
c if SSB is below 1/10 of Blim      
      if (truessb(iy,iter).lt.blim/10.0 
     &.or.jflg.eq.1) iflag(iy,iter)=1
c project into next year      
      if (iy.lt.maxyr) then
c The actual recruitment parameters may be changed
      call rcrparam(iy,1)
      call updatewema(iy,rn)
      call nextyear(rn,truef(iy,iter),s,sigmar,2) 
      call obsrecruit(iy,rn(0),cvrecrsurv)
      call obssurvey(iy,rn,truef(iy,iter),cvbiosurv,2)


c Note the yearclass factor for weights for the new year class
      wyclfct(iy+1)=wyclfct0
      do 22 ia=0,iaold
      weca(ia)=trueweca(iy+1,ia)
      west(ia)=truewest(iy+1,ia)
      rmat(ia)=truemat(iy+1,ia)
22    truen(ia,iy+1)=rn(ia)
      truerecr(iy+1,iter)=rn(0)


       endif
c end year loop

20    continue

c print things that are discarded after each iteraion
      if (ioutopt(19).eq.1.and.iter.le.100) call printbas(truen)
      if (ioutopt(20).eq.1.and.iter.lt.100) call printsr(truen)
      if (ioutopt(25).eq.1.and.iter.lt.10) call printwema
      if (ioutopt(26).eq.1.and.iter.lt.20) call printrecr
c end iteration loop
999   continue
      end


      subroutine hcr(tactemp,iy)
c generate TACs for year iy (and further if multiannual TACs)
c in tactemp, based on obssurviv and weights and maturities in the last assessment year
      include 'hcscom13_2.i'
c Local variables.
      real rn(0:maxage),rny(0:maxage),
     &tactemp(0:maxyrp),tacout(0:maxyrp),
     &tacprev(0:maxyrp),tempc(0:maxyrp),
     &basis(mbasis)
c temporary TACs are defined maxyrp beyond last year, to allow projections that far.     
      integer iy,iflg
c fill in the working array sy and tactemp with old values as far as they go      
      do 40 i=0,iy-1
      tactemp(i)=tac(i,iter)
      tacout(i)=0.0
40    continue  
c Prepare initial values etc.
      do 10 i=iy,iy+iproj
      tactemp(i)=tactemp(i-1)
10    continue
      do 133 i=iy+lastassn,iy+iproj
      tacprev(i)=tactemp(i)
      tacout(i)=0.0
133    continue   
      do 11 ia=0,iaold
      rn(ia)=rnobs(ia,iy+lastassn,iy+lastassn)
11    continue
c set weights and maturities to the last assessment year
      do ia=lowage,iaold
      weca(ia)=trueweca(iy+lastassn,ia)
      west(ia)=truewest(iy+lastassn,ia)
      rmat(ia)=truemat(iy+lastassn,ia)
      enddo      

      nr=0
      v1=0.0
c Step 1: apply basic rule

c Project the stock forwards with 
c unconstrained TAC according to rule (call to rule and translation)
c      write(*,*) 'call project',iter
100   call project(rn,rny,tactemp,tacout,iy,iflg)
c Build basis  
c      write(*,*) 'call buildbasis'
      call buildbasis(iy,basis)
c      write(*,*) 'call rule'

      call rule(v,basis)

      nr=nr+1
c v2 and v1 keep previous values of v.
c Used in the searching routine below,
c if cyclic iterations are suspected.
      v2=v1
      v1=v
      
      
c translate this v into TAC      
      if (ibasrule.eq.1) then
c case v is an F-value
c      write(*,*) 'call yield'

      call yield(rny,y,v,1)
      
c in this case, keep the decided F for the record
      obsf(iy,iter)=v
      else if (ibasrule.eq.2) then
c case v is a HR rule based  on an assessment
       call calcbplus(rny,tsb)
       y=v1*tsb
      else if (ibasrule.eq.3) then
c case v is a HR, based on the basis1 for decision
       y=v1*basis(1)
      else if (ibasrule.eq.4) then
c case v is a TAC rule
      y=v1
      endif
c      if (iy.ge.maxyr-1) write(*,*) 'tacmax',iy,v,
c     &basis(1),basis(2),tactemp(iy),weca(5)


c set the decided TAC for the whole decision period
      tactemp(iy)=y
      primtac=y
      step=(tactemp(iy)-tactemp(iy-1))/float(idur)
      do  i=iy,iy+idur-1
      if (idur.gt.1.and.iabrgrad.eq.2) then
      tactemp(i)=tactemp(i-1)+step
      else
      tactemp(i)=tactemp(iy)
      endif
      enddo
      


      if (iproj.gt.0) then
c Case SSB or Bplus will depend on TAC 
c check for change in TAC, and iterate if there is a change     
      ssq=0.0
      do 14 i=iy,iy+idur-1
      ssq=ssq+(tactemp(i)-tacprev(i))**2
14    continue
      if (ssq.gt.0.0001) then
      do 15 i=iy,iy+idur-1
      tacprev(i)=tactemp(i)
15    continue
      if (nr.lt.10) then
      goto 100
      else
      
c If the iterations have gone too far, do a search for the critical point instead
      if (v1.gt.v2) then
      vx=v1
      v1=v2
      v2=vx
      endif
      deltav=(v2-v1)/10.0
110   v1=v1+deltav
c translate this v into TAC      
      if (ibasrule.eq.1) then
c case v is an F-value
      call yield(rny,y,v1,1)
c in this case, keep the decided F for the record
      obsf(iy,iter)=v1
      else if (ibasrule.eq.2) then
c case v is a HR rule
c corrected 28/10-12      
       call calcbplus(rny,tsb)
       y=v1*tsb
      else if (ibasrule.eq.3) then
c case v is a TAC rule
      y=v1
      endif
c insert the y:
      tactemp(iy)=y
c We now have the primary TAC for year iy
      primtac=y

c set the decided TAC for the whole decision period
      step=(tactemp(iy)-tactemp(iy-1))/float(idur)
      do  i=iy,iy+idur-1
      if (idur.gt.1.and.iabrgrad.eq.2) then
      tactemp(i)=tactemp(i-1)+step
      else
      tactemp(i)=tactemp(iy)
      endif
      enddo
c check that it is stable
      call project(rn,rny,tactemp,tacout,iy,iflg)
       call rule(v2,basis)
      if (v2.gt.v1) then
      goto 110
      else
      if (abs(deltav).gt.ruletarget(iy)*0.00001) then
      deltav=deltav*0.1
      v1=v2-deltav
      goto 110
      endif
      endif
      endif
c end search. The current sy and tactemp are associated with the v 
c that reproduces itself      
      endif
c end (if ssq > small)      
      endif     
c end iteration needed
c We now have the TAC according to the basic rule in tactemp.

c Step 2: Apply stabilizers
c 1. Apply filter if asked for
      if (hcrparam(11).lt.1.0.and.iy.gt.0) then
c Put the filtered tacs in tempc and check
      i1=iy+icbastable(1,2)
      i2=iy+max(icbastable(1,3),idur)
      tempc(i1-1)=tactemp(i1-1)
      do i=i1,i2
      if (i.lt.iy) then
      tempc(i)=tactemp(i)
      else
      tempc(i)=tempc(i-1)*(1.0-hcrparam(11))+
     &tactemp(i)*hcrparam(11)
      endif
      enddo
c then check the criterium
      call project(rn,rny,tempc,tacout,iy,iflg)
      call buildcbasis(cbasis,1,iy)
c Shall we use the filter, now represented in tempc?
      if (icbastable(1,4).eq.3) then
c if the cbasis is a trend, use filter if the trend is smaller than the criterium
      if (cbasis.lt.hcrparam(12)) then
      do i=iy,iy+idur
      tactemp(i)=tempc(i)
      enddo
      endif
      else
c in other cases, use the filter if the cbasis is above the criterium
      if (cbasis.gt.hcrparam(12)) then
      do i=iy,iy+idur
      tactemp(i)=tempc(i)
      enddo
      endif
c end filter asked for
      endif

c this is the criterium for applying filter
      tactemp(iy)=tactemp(iy-1)*(1.0-hcrparam(11))+
     &tactemp(iy)*hcrparam(11)
      endif
c If this TAC is to be valid for several years, set it for the whole period
c before checking the percentage rule.
c Note that it is the filtered TAC that is the reference here.
      if (idur.gt.1) then
      if (iabrgrad.eq.1) then
      do i=iy,iy+idur
      tactemp(i)=tactemp(iy)
      enddo
      else if (iabrgrad.eq.2) then
      diff=(tactemp(iy)-tactemp(iy-1))/float(idur)
      do i=iy,iy+idur
      tactemp(i)=tactemp(i-1)+diff
      enddo
      endif
      endif

c 2. Does percentage constraint apply?
      percent=hcrparam(13)/100.0
      if (percent.gt.0.0.and.iy.gt.0) then     
      actratio=tactemp(iy)/tactemp(iy-1)-1.0      
        if (abs(actratio).gt.percent) then
c Change is big enough
c cratio is the multiplier for change
         if (actratio.lt.0.0) then
           cratio=(1.0-percent)
         else
           cratio=(1.0+percent)      
         endif
c Derogation?
        if (ialwonly.eq.1) then
c No, apply always. If the change is bigger than the gradual change,
c the gradual change is overrided.
        tactemp(iy)=tactemp(iy-1)*cratio
        if (idur.gt.1) then
        do i=iy+1,iy+idur
        tactemp(i)=tactemp(i-1)*cratio
        enddo
        endif
        
        else if (ialwonly.eq.2) then
c Yes, case only: Does the constrained TAC lead above Btrig?
c To calculate criterium, we need a projection covering the case with 
c multiannual TAC
c First, move the constrained TAC to tempc
      i1=iy+icbastable(2,2)
      i2=iy+max(icbastable(2,3),idur)
      tempc(i1-1)=tactemp(i1-1)
      do i=i1,i2
      if (i.lt.iy) then
      tempc(i)=tactemp(i)
      else
      tempc(i)=tempc(i-1)*cratio
      endif
      enddo
c then check the criterium
      call project(rn,rny,tempc,tacout,iy,iflg)
      call buildcbasis(cbasis,2,iy)
c Use tempc (=constrained tac) if outside criterium
      if (icbastable(2,4).eq.3) then
c if the cbasis is a trend,, use constraint if the trend is smaller than the criterium
      if (cbasis.lt.hcrparam(14)) then
      do i=iy,iy+idur
      tactemp(i)=tempc(i)
      enddo
      endif
      else
c in other cases, use the constraint if the cbasis is above the criterium
      if (cbasis.gt.hcrparam(14)) then
      do i=iy,iy+idur
      tactemp(i)=tempc(i)
      enddo
      endif
      endif
c end if ialwonly=2
      endif
   
c end if actratio > cratio 
       endif
c end if percentage >0
       endif
c Check min and max TAC
      do i=iy,iy+idur-1
      if (hcrparam(15).gt.0.0) tactemp(i)=max(tactemp(i),hcrparam(15)) 
      if (hcrparam(16).gt.0.0) tactemp(i)=min(tactemp(i),hcrparam(16))
      enddo
 
c reset weights and maturities
      do ia=lowage,iaold
      weca(ia)=trueweca(iy,ia)
      west(ia)=truewest(iy,ia)
      rmat(ia)=truemat(iy,ia)
      enddo
c keep basis1 for printout
      do i=1,nbasis
      decbas(iy,i,iter)=basis(i)
      enddo
      end       
       
      subroutine rule(v,basis)
      include 'hcscom13_2.i'
      real v, basis(mbasis)
      if (irule.eq.1) then
      call rule1(v,basis)
      else if (irule.eq.2) then
      call rule2(v,basis)
      else if (irule.eq.3) then
      call rule3(v,basis)
      else if (irule.eq.4) then
      call rule4(v,basis)
      else if (irule.eq.5) then
      call rule5(v,basis)
      else if (irule.eq.6) then
      call rule6(v,basis)
      else if (irule.eq.7) then
      call rule7(v,basis)     
      else if (irule.eq.8) then
      call rule8(v,basis)
      else
      write(*,*) 'Unknown harvest rule',irule
      stop
      endif
      end

                  
                   
      subroutine rule1(v,basis)
      include 'hcscom13_2.i'
c alfa rule 
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      vstd=hcrparam(3)
      alfa1=hcrparam(4)
      alfa2=hcrparam(5)
      v=vstd
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
         v=vstd*(1.0-alfa1*(btrig1-basis(1))/btrig1)
        else
         v=vstd
        endif
       else if (basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
         v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
        else
         v=vstd
        endif
       endif
       if (v.lt.0.0) v=0.0
       end


      subroutine rule2(v,basis)
      include 'hcscom13_2.i'
      real v,basis(mbasis)
c slope between Btrig1 and B0, flat below
c Both b0 and btrig1 relate to basis(1)
      b0=hcrparam(1)
      btrig1=hcrparam(2)
      btrig2=hcrparam(3)
      vstd=hcrparam(4)
      vlevel1=hcrparam(5)
      alfa2=hcrparam(6)
      if (basis(1).lt.b0) then
      v=vlevel1
      else if (basis(1).lt.btrig1) then
      v=vlevel1+(vstd-vlevel1)*(basis(1)-b0)/(btrig1-b0)
      else if (basis(2).lt.btrig2) then
      v=vstd
      else
      v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
c      write(*,*) 'rule',basis(1),basis(2),vstd,v
      endif
      end
      

      subroutine rule3(v,basis)
      include 'hcscom13_2.i'
c Like rule 1, but:
c vstd depends on basis(3), with a factor gain
c Basis(3) is presumably a relative slope
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      alfa1=hcrparam(4)
      alfa2=hcrparam(5)
      gain=hcrparam(6)
      vstd=hcrparam(3)*(1.0+gain*basis(3))
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
         v=vstd*(1.0-alfa1*(btrig1-basis(1))/btrig1)
        else
         v=vstd
        endif
       else if (basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
         v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
        else
         v=vstd
        endif
       endif
       if (v.lt.0.0) v=0.0
       end


      subroutine rule4(v,basis)
c Upper part of logistic function below B1 and above B2
      include 'hcscom13_2.i'
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      slope1=hcrparam(4)
      slope2=hcrparam(5)
      vstd=hcrparam(3)
      vmax=hcrparam(6)
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
        vact=1.0/(1.0+exp(-slope1*(basis(1)/btrig1)))
        v0=1.0/(1.0+exp(-slope1))-0.5
        v=vstd*(vact-0.5)/v0
        else
         v=vstd
        endif
      else if (basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
        vact=1.0/(1.0+exp(-slope2*(basis(2)-btrig2)/btrig2))
        v=vstd+(vact-0.5)*(vmax-vstd)
c        write(*,*) basis(2),vact,vmax,vstd,v
        else
         v=vstd
        endif
       if (v.lt.0.0) v=0.0
       else
       v=vstd
       endif
      if (v.lt.0.0) v=0.0

      end

      subroutine rule5(v,basis)
      include 'hcscom13_2.i'
c alfa rule with variable vstd
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      refr=hcrparam(3)
      alfa1=hcrparam(4)
      alfa2=hcrparam(5)
      vlow=hcrparam(6)
      vhigh=hcrparam(7)
      if (basis(3).gt.refr) then
      vstd=vhigh
      else
      vstd=vlow
      endif
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
         v=vstd*(1.0-alfa1*(btrig1-basis(1))/btrig1)
        else
         v=vstd
        endif
       else if (basis(2).gt.btrig2) then
        if (btrig2.gt.0) then
         v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
        else
         v=vstd
        endif
       endif
       if (v.lt.0.0) v=0.0
c      write(*,*) 'rule5',basis(1),basis(2),basis(3),vstd,v
       end

      subroutine rule6(v,basis)
      include 'hcscom13_2.i'
      real v,basis(mbasis)
c slope between Btrig1 and B0, flat below
c Both b0 and btrig1 relate to basis(1)
c Variable vstd
      b0=hcrparam(1)
      btrig1=hcrparam(2)
      btrig2=hcrparam(3)
      refr=hcrparam(4)
      alfa2=hcrparam(5)
      vlevel1=hcrparam(6)
      vlow=hcrparam(7)
      vhigh=hcrparam(8)
      if (basis(3).gt.refr) then
      vstd=vhigh
      else
      vstd=vlow
      endif
      
      if (basis(1).lt.b0) then
      v=vlevel1
      else if (basis(1).lt.btrig1) then
      v=vlevel1+(vstd-vlevel1)*(basis(1)-b0)/(btrig1-b0)
      else if (basis(2).lt.btrig2) then
      v=vstd
      else
      v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
c      write(*,*) 'v',vstd,v,basis(3),refr
      endif
      end

      subroutine rule7(v,basis)
c Sardine-rule
      include 'hcscom13_2.i'
      real v,basis(mbasis)
      
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      vstd=hcrparam(3)
      alfa1=hcrparam(4)
      alfa2=hcrparam(5)

      if (basis(1).gt.btrig2) then
      v=vstd
      else if (basis(1).lt.btrig2.and.basis(1).gt.btrig1) then
       if (btrig2.gt.btrig1) then
        v=alfa2+(basis(1)-btrig1)/(btrig2-btrig1)*(vstd-alfa2)
       else
        v=vstd
       endif
      else if (basis(1).lt.btrig1) then
         v=alfa2*(1.0-alfa1*(btrig1-basis(1))/btrig1)
         if (v.lt.0.0) v=0.0
      endif
      write(*,*) basis(1),btrig1,btrig2,vstd,alfa1,alfa2,v
      end
 
       subroutine rule8(v,basis)
      include 'hcscom13_2.i'
c alfa rule with variable vstd
      real v,basis(mbasis)
      btrig1=hcrparam(1)
      btrig2=hcrparam(2)
      vstd=hcrparam(3)
      alfa1=hcrparam(4)
      alfa2=hcrparam(5)
      vlow=hcrparam(6)
      vhigh=hcrparam(7)
      if (basis(1).lt.btrig1) then
        if (btrig1.gt.0.0) then
         v=vstd*(1.0-alfa1*(btrig1-basis(1))/btrig1)
        else
         v=vstd
        endif
      else 
        if (basis(2).gt.btrig2.and.btrig2.gt.0.0) then
         v=vstd*(1.0+alfa2*(basis(2)-btrig2)/btrig2)
        else
         v=vstd
        endif
      endif
       if (v.lt.vlow) v=vlow
       if (v.gt.vhigh) v=vhigh
!       write(*,*) basis(1),btrig1,vstd,vlow,vhigh,v
       end

      
      subroutine buildbasis(iy,basis)
c build the decision basis vector (in basis) for the TAC for year iy
c bas(year) is the intermediate value for the basis type in the year.
      include 'hcscom13_2.i'
      real x(niter),y(niter),
     &rn(0:maxage),bas(-maxage:maxyr+mbasis),basis(1:mbasis)
      integer ibas
c Outer loop over elements in basis bector (ibas)
      do ibas=1,nbasis
      i1=ibastable(ibas,2)
      i2=ibastable(ibas,3)
      do i=iy+i1,iy+i2
c      write(*,*) 'years',iy,i,i1,i2
c Basis is derived from stocknumbers, so they are assembled first
c get the stock numbers from the right source, but only if needed
      if (ibastable(ibas,1).le.3) then
      do ia=lowage,iaold
      if (i.lt.iy+lastassn) then
      rn(ia)=rnobs(ia,i,iy+lastassn)
      else
      rn(ia)=rnproj(ia,i)
      endif
      enddo
      endif
c caclulate actual basis values for year i
      if (ibastable(ibas,1).eq.1) then
      
      call calcssb(rn,s,fobs(i),1)
      bas(i)=s
c      write(*,*) 's',iy,i,s,fobs(i)
      else if (ibastable(ibas,1).eq.2) then
      call calcbplus(rn,bs)
      bas(i)=bs
      else if (ibastable(ibas,1).eq.3) then
       bas(i)=rn(lowage)
      else if (ibastable(ibas,1).eq.4) then
      bas(i)=recrsurv(i)
      else if (ibastable(ibas,1).eq.5) then
      if (biosurv(i,1).lt.0.0) then
      write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
      stop
      else
      bas(i)=biosurv(i,1)
      endif
      else if (ibastable(ibas,1).eq.6) then
      if (biosurv(i,2).lt.0.0) then
      write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
      stop
      else
      bas(i)=biosurv(i,2)
      endif
      endif
      enddo
c End of year loop, values for year i in bas(i)
c Calculate basis according to type     
      if (ibastable(ibas,4).eq.1) then
c mean
      s=0.0
      do j=iy+i1,iy+i2
      s=s+bas(j)
      enddo
      basis(ibas)=s/float(ibastable(ibas,3)-ibastable(ibas,2)+1)
      else if (ibastable(ibas,4).eq.2) then
c minimum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).lt.s) s=bas(j)
      enddo
      basis(ibas)=s
      else if (ibastable(ibas,4).eq.3) then
c linear trend
      if (i2-i1.lt.1) then
      write(*,*) 'You ask for a trend over less than two years'
      write(*,*) 'Check definition of basis on the opt.inn file'
      stop
      endif
      do j=iy+i1,iy+i2
      jj=j-(iy+i1)+1
      x(jj)=float(jj)
      y(jj)=bas(j)
      enddo
      n=ibastable(ibas,3)-ibastable(ibas,2)+1
      call regress (x,y,a,b,n,ymean)
      if (ymean.gt.0.0) then
      basis(ibas)=b/ymean
      else
      basis(ibas)=0.0
      endif
      endif
      enddo
c      write(*,*) 'basis1',basis(1)
      end



      subroutine compbasis(truen)
c calculate the decision basis vector
c with true Ns, for the printoutroutine printbas
      include 'hcscom13_2.i'
      real x(niter),y(niter),
     &rn(0:maxage),bas(0:maxyr),truen(0:maxage,0:maxyr)
      do ibas=1,nbasis
      i1=ibastable(ibas,2)
      i2=ibastable(ibas,3)
      do iy=1,maxyr
c Only years where the whole range is covered
      if (iy+i1.ge.0.and.iy+i2.le.maxyr) then
      do i=iy+i1,iy+i2
      do ia=lowage,iaold
      rn(ia)=truen(ia,i)
c      write(*,*) i1,i2,iy,i,ia,truen(ia,i)
      enddo
c caclulate actual basis values
      if (ibastable(ibas,1).eq.1.or.ibastable(ibas,1).eq.5) then
      call calcssb(rn,s,fobs(i),1)
      bas(i)=s
      else if (ibastable(ibas,1).eq.2.or.ibastable(ibas,1).eq.6) then
      call calcbplus(rn,bs)
      bas(i)=bs
      else if (ibastable(ibas,1).eq.3) then
      bas(i)=recrsurv(i)
      else if (ibastable(ibas,1).eq.4) then
      bas(i)=rn(lowage)
      endif
      enddo

c Calculate basis according to type      
      if (ibastable(ibas,4).eq.1) then
c mean
      s=0.0
      do j=iy+i1,iy+i2
      s=s+bas(j)
      enddo
      truebas(ibas,iy)=s/float(ibastable(ibas,3)-ibastable(ibas,2)+1)
      else if (ibastable(ibas,4).eq.2) then
c minimum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).lt.s) s=bas(j)
      enddo
      truebas(ibas,iy)=s
      else if (ibastable(ibas,4).eq.3) then
c linear trend
      if (i2-i1.lt.1) then
      write(*,*) 'You ask for a trend over less than two years'
      write(*,*) 'Check definition of basis on the opt.inn file'

      stop
      endif
      do j=iy+i1,iy+i2
      jj=j-(iy+i1)+1
      x(jj)=float(jj)
      y(jj)=bas(j)
      enddo
      n=ibastable(ibas,3)-ibastable(ibas,2)+1
      call regress (x,y,a,b,n,ymean)
      if (ymean.gt.0.0) then
      truebas(ibas,iy)=b/ymean
      else
      truebas(ibas,iy)=0.0
      endif
      endif
      else
      truebas(ibas,iy)=-1.0
      endif
      enddo
      enddo
      end





      
      subroutine buildcbasis(cbasis,icnr,iy)
c build the decision basis vector for the constraint on TAC for year iy,
c icnr=1 is for the filter rule, icnr=2 is for the percentage rule
c cbasis is a single number, the constrint rules do not operate with a vector of bases
      include 'hcscom13_2.i'
      real cbasis,x(niter),y(niter),rn(0:maxage),
     &bas(-maxage:maxyr+mbasis)
c find sequence of ssb and tsb
      i1=icbastable(icnr,2)
      i2=icbastable(icnr,3)
      do i=iy+i1,iy+i2
c get the stock numbers from the right source, if needed
       if (icbastable(icnr,1).le.3) then
       do ia=lowage,iaold
       if (i.lt.iy+lastassn) then
        rn(ia)=rnobs(ia,i,iy+lastassn)
       else
        rn(ia)=rnproj(ia,i)
       endif
       enddo
       endif
c then we can calculate the basis for year i       
       if (icbastable(icnr,1).eq.1) then
        call calcssb(rn,s,fobs(i),1)
        bas(i)=s
       else if (icbastable(icnr,1).eq.2) then
        call calcbplus(rn,bs)
        bas(i)=bs
       else if (icbastable(icnr,1).eq.3) then
        bas(i)=rn(lowage)
       else if (icbastable(icnr,1).eq.4) then
        bas(i)=recrsurv(i)
       else if (icbastable(icnr,1).eq.5) then
        if (biosurv(i,1).lt.0.0) then
         write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
         stop
        else
         bas(i)=biosurv(i,1)
        endif
       else if (icbastable(icnr,1).eq.6) then
        if (biosurv(i,2).lt.0.0) then
         write(*,*) 'A biomass survey is included in the decision basis'
      write(*,*) 'in year TAC year + ',i,'which is before it has been pe
     &rformed'
      write(*,*) 'Correct the specification of the basis in the opt.inn 
     &file'
         stop
        else
         bas(i)=biosurv(i,2)
        endif
       endif
      enddo
c end year loop, bas(i) has the basis for year i
 
c Calculate basis according to type      
      if (icbastable(icnr,4).eq.1) then
c mean
      s=0.0
      do j=iy+i1,iy+i2
      s=s+bas(j)
      enddo
      cbasis=s/float(icbastable(icnr,3)-icbastable(icnr,2)+1)
      else if (icbastable(icnr,4).eq.2) then
c minimum
      s=bas(iy+i1)
      do j=iy+i1,iy+i2
      if (bas(j).lt.s) s=bas(j)
      enddo
      cbasis=s
      else if (icbastable(icnr,4).eq.3) then
c linear trend
      if (i2-i1.lt.1) then
      write(*,*) 'You ask for a trend over less than two years'
      write(*,*) 'Check definition of basis on the opt.inn file'
      stop
      endif
      do j=iy+i1,iy+i2
      jj=j-(iy+i1)+1
      x(jj)=float(jj)
      y(jj)=bas(j)
      enddo
      n=icbastable(icnr,3)-icbastable(icnr,2)+1
      call regress (x,y,a,b,n,ymean)
      if (ymean.gt.0.0) then
c Absolute trend in this case
      cbasis=abs(b/ymean)
      else
      cbasis=0.0
      endif
      endif
      end


      subroutine project(rninit,rny,tacinn,
     &tacout,iy,iflg)

c Deterministic projection presumed to start in year iy+lastassn and continuing to iy+iproj      
c Assumed TACs are tacinn. 
c Tacout holds TACs that come from the findf subroutine
c They will deviate from tacinn if the tac in tacinn cannot be taken
c Recruitments are according to the deterministic SR function.
c It is assumed that rninit is the number at start of iy+lastassn
c The results of the projection is in the common rnproj, and for the year iy in rny
c  
      include 'hcscom13_2.i'
      real rninit(0:maxage),rn1(0:maxage),
     &tacinn(0:maxyrp),tacout(0:maxyrp),rny(0:maxage)
      integer iy,iflg,iflg1
      iflg=0
      do 10 ia=0,iaold
      rn1(ia)=rninit(ia)
c rnproj always holds the starting numbers 
      rnproj(ia,iy+lastassn)=rn1(ia)
10    continue
      if ((iproj).gt.0.or.lastassn.lt.0) then
c projection relevant      
       do 20 i=iy+lastassn,iy+iproj
c       write(*,*) 'project',iy,i
       if (i.le.(iy+idur-1)) then
c i is a year with at least preliminary TAC      
        tacout(i)=tacinn(i)
        call findf(rn1,tacout(i),f,iflg1,1)
        if (iflg1.eq.1) iflg=1
       else
c i is a year after the last TAC. Assume same f      
        call yield(rn1,y,f,1)
        tacout(i)=y
       endif
       if (i.le.maxyr) fobs(i)=f
       call calcssb(rn1,s,f,1)
       sr=0.0
       if (i.eq.iy) then
        do 30 j=0,iaold
        rny(j)=rn1(j)
30    continue 
       endif     
       if (i.le.iy+iproj) then
        call nextyear(rn1,f,s,sr,1)
c The recruitment in rn1 shall be the assumed, not the one derived in nextyear   
c Two alternatives for recruitment, signalled by nrecryears
c nrecryears=0:
c The parameters that will be seen in the recruitment routine are found by rcrparam,
c and are those for the year iy+lastassn, i.e. the regime valid in the intermediate year 
c Flag 0 in the call to rcrparam and ps=0: no stochastic noise)
c nrecrparam>0:
c take the mean of percieved recruitments
        if (nrecryears.eq.0) then
        call rcrparam(iy+lastassn,0)
        ps=0.0
        call recr(r0,s,ps)
        else
        call getmeanrecr(iy,r0,nrecryears)
        endif
        do ia=0,lowage
         rn1(ia)=r0
c         write(*,*) 'recr',ia,iy,rn1(ia)
        enddo
       do ia=0,iaold
       if (i.lt.maxyr+mbasis) rnproj(ia,i+1)=rn1(ia)
       enddo
       endif
c Keep recruitment for comparison
      if (i.eq.iy) obsrecr(iy,iter)=rn1(lowage)

20    continue
      else
c No projection - transfer initial numbers to rny
      do ia=0,iaold
      rny(ia)=rn1(ia)
      enddo
      obsrecr(iy,iter)=rn1(lowage)
      endif
      end      
            
            
      subroutine getmeanrecr(iy,r0,n)
c Get the recruitment in projection as the mean of percieved recruitments over n years      
      include 'hcscom13_2.i'
      integer iy,n
      real r0
c in the call, n hardcoded so far,
c The modified iobsstart in line 509 must be activated
c NOTE: The youngest age in rnobs is lowage.
      r0=0.0
      do i= iy+lastassn-n+1,iy+lastassn
      r0=r0+rnobs(lowage,i,iy+lastassn)
c      write(*,*) i,iy,rnobs(lowage,i,iy+lastassn)
      enddo
      r0=r0/float(n)
c      write(*,*) 'meanrec',r0
      end
     
     
      subroutine obsmodel(rnold,iy,iback)
c converts the real stock numbers to corrupted stock numbers
c Input is current rn at age in rnold, output in common rnobs 
c This is the history as percieved in year iy, backcalculated from the start of iy.
c In addition, it updates the correspondning F-values representing the last assessment
c It also holds the newly drawn xsi-values in xsiobs, that are used
c for calculating the xsi to be applied as a autoregressive process
      include 'hcscom13_2.i'
      real rnold(0:maxage)
      integer iy,iback
      biasobs= hcrparam(18)
      yfctobs= hcrparam(20)
c build a new vector of random values.
c btemp is the common random multiplier
c if iy=0, the xsi matrix has to be built
      if (iy.eq.0) then
      istart=-nac 
      else
      istart=iy
      endif

      do i=istart,iy
      x=rnx(rintvar)
      xsi=snrn(x)*yfctobs
      btemp=(1.0+biasobs)*exp(xsi-yfctobs**2/2.0)
c By age
      do ia=0,iaold
      if (ia.lt.lowage) then
      rnobs(ia,iy,iy)=rnold(ia)
      else
      if (cvobs(ia,1).gt.0.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cvobs(ia,1)
      else
      xsi=0.0
      endif
      xsiobs(ia,i)=btemp*exp(xsi-cvobs(ia,1)**2/2.0)
      endif
      enddo
      enddo
c get the actual xsi-value for the current year
      do ia=0,iaold
      a=0.0
      xsiact=0.0
      do i=0,nac
      if (ia-i.ge.lowage) then
      xsiact=xsiact+acfact(i)*xsiobs(ia-i,iy-i)
      a=a+acfact(i)
      endif
      enddo
      if (a.gt.0.0) then
      xsiact=xsiact/a
      else
      xsiact=1.0
      endif
c Apply to n-values in the last year
      rnobs(ia,iy,iy)=rnold(ia)*xsiact
c      write(*,*) iy,ia,rnobs(ia,iy,iy),rnold(ia),xsiact
      enddo

      if (iback.gt.0) then
c VPA backwards until iy+iobsstart (if asked for: iback>0)

c The cohorts that has age ia in year iy
      do ia=lowage,iaold
      do i= 1,-iobsstart
       if (ia-i.ge.lowage) then
      rnobs(ia-i,iy-i,iy)= rnobs(ia-i+1,iy-i+1,iy)*exp(rm(ia-i))+
     &cobs(ia-i,iy-i)*exp(rm(ia-i)/2.0)
c      write(*,*) 'step1',iy,ia,i,iy-i,ia-i,rnobs(ia-i,iy-i,iy)
c      write(*,*) rnobs(ia-i+1,iy-i+1,iy),rm(ia-i),cobs(ia-i,iy-i)
      endif
      enddo
       enddo
c year classes that start at oldest true age in year j 
      do j=iy-1,iy+iobsstart,-1
c rn in year j in that year class is derived from the F in the year class before
c and the catch of the actual year class
c Find F for the year class having penultimate age in the same year
      ztemp=log(rnobs(iaold-2,j,iy)/rnobs(iaold-1,j+1,iy))
      ftemp=ztemp-rm(iaold-1)
c apply that F to the catch in year j to get rnobs(iaold-1,j,iy)
      rnobs(iaold-1,j,iy)= 
     &cobs(iaold-1,j)*ztemp/ftemp/(1.0-exp(-ztemp))  
c calculate backwards, i is counting backwards
c j-i is the year
      do i=1,-iobsstart
      ia = iaold-i-1
      if (ia.ge.lowage.and.j-i.ge.iy+iobsstart) then
      rnobs(ia,j-i,iy) = rnobs(ia+1,j-i+1,iy)*
     &exp(rm(ia))+cobs(ia,j-i)*exp(rm(ia)/2.0)
c     write(*,*) 'rn1',iy,j-i,ia,rnobs(ia,j-i,iy),rnobs(ia+1,j-i+1,iy),
c     &cobs(ia,j-i)
      endif
       enddo
      enddo
c Find average F according to n-values from obsmodel in the current assessment.
      do i=iy+iobsstart,iy
      ftemp=0.0
      do ia=iaref1,iaref2
      ftemp=ftemp+log(rnobs(ia-1,i,iy)/rnobs(ia,i+1,iy))-rm(ia)
      enddo
      fobs(i)=ftemp/float(iaref2-iaref1+1)
c      write(*,*) 'fobs in obsmod',iy,i,fobs(iy-i)
      enddo
      endif
      end


      subroutine getobscanum(canum,iy)
      include 'hcscom13_2.i'
      real canum(0:maxage)
      integer iy
      do ia=0,maxage
      cobs(ia,iy)=0.0
      enddo
      do ia=lowage,iaold
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cvobs(ia,2)
      cobs(ia,iy)=canum(ia)*exp(xsi)
c      write(*,*) 'obscanum',iy,ia,canum(ia),cobs(ia,iy)
      enddo
      end


      subroutine obsrecruit(iy,r,cv)
c Imitate a recruitment survey, based on actual recruitment
c to be called just after nextyear in bootruns
      include 'hcscom13_2.i'
      real r,cv
      integer iy
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cv
      recrsurv(iy)=r*exp(xsi)
      end


      subroutine obssurvey(iy,rn,fact,cv,isel)
c Imitate a biomass survey,by corrupting true biomass
c to be called just after nextyear in bootruns
      include 'hcscom13_2.i'
      real rn(0:maxage),fact,cv,s,bs
      integer iy,isel
      call calcssb(rn,s,fact,isel)
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cv
      biosurv(iy,1)=s*exp(xsi)

      call calcbplus(rn,bs)
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cv
      biosurv(iy,2)=bs*exp(xsi)
c      write(*,*) 'biosurv',iy,bs,biosurv(iy,1),biosurv(iy,2)
      end


      subroutine implement(yc,yci,rn,f,jflg,canum)
      include 'hcscom13_2.i'
c Adds implemetation error to the presumed catch in numbers.
c calculates the true f at age and common f
c Input: yc: Agreeed TAC
c        rn: True stock numbers
c Output: yci: realized catch
c         f: true f-factor
c         canum: actual removal at age
c as common: sel(ia,2): true selection     
c Revised in version 12_3     
      real yc,yci,f,canum(0:maxage),rn(0:maxage),fa(0:maxage)
      integer isel,jflg
c isel: 1: Standard selection, 2. true selection after implementation)
      biasimp= hcrparam(19)
      yfctimp= hcrparam(21)
c Transform TAC to canum assuming standard selection:
      isel=1
      jflg=0
      call findf(rn,yc,ftemp,jflg,isel)
      do 11 ia=0,iaold
      if (ia.lt.lowage) then
      canum(ia)=0.0
      else
      z=ftemp*sel(ia,isel)+rm(ia)
      if (z.gt.0.0) then
      canum(ia)=ftemp*sel(ia,isel)*rn(ia)*(1.0-exp(-z))/z
      else
      canum(ia)=0.0
      endif
      endif
11    continue
c apply noise to canum
c find the yearfactor common to all ages (biased random)
c gives yci: which shall be realized SOP
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*yfctimp
      btemp=(1.0+biasimp)*exp(xsi)
      yci=yc*btemp
c find age factor for each age and apply, gives noisy and biased canum      
      do 10 ia=lowage,iaold
      if (cvimp(ia).gt.0.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xsi=xsi*cvimp(ia)
      else
      xsi=0.0
      endif
      canum(ia)=canum(ia)*exp(xsi)*btemp
      if (canum(ia).le.0.0) canum(ia)=0.0
 
10    continue
c translate canum to yield (SOP)      
      yctemp=0.0
      do 23 ia=lowage,iaold
      yctemp=yctemp+canum(ia)*weca(ia)
23    continue
c adjust canum to get the right realized SOP
      do ia=lowage,iaold
      canum(ia)=canum(ia)*yci/yctemp
c     write(*,*) 'canum',ia,canum(ia)
      enddo
c Find true F corresponding to canum at each age
      do 20 ia=lowage,iaold
      if (canum(ia).le.0.0) then
      fa(ia)=0.0
      else
      ef=1.0-canum(ia)/rn(ia)*exp(rm(ia)/2.0)
c protection if the stock is not sufficient for the catch at an individual age
      if (ef.gt.0.0) then
      fa(ia)=-log(ef)
c      write(*,*) ia,ef,fa(ia)
      else
      fa(ia)=fmax
      jflg=1
      endif
c      write(*,*) 'fa',ia,fa(ia)
      endif
20    continue
c Scale fa to get true selection (isel=2)
      f=0.0
      do 22 ia=iaref1,iaref2
      f=f+fa(ia)
22    continue
      f=f/float(iaref2-iaref1+1)
      do 21 ia=lowage,iaold
      if (f.gt.0.0) then
      sel(ia,2)=fa(ia)/f
      else
      sel(ia,2)=0.0
      endif
21    continue 

c get true F level giving the true sop with this selection
      call findf(rn,yci,f,jflg,2)
      end      
      
      subroutine findf(rn,yy,fact,jflg,isel)
c find f-multiplier=fact corresponding to yield yy  
c If that is not achievable within f = fmax, 
c fact=fmax and yy=yield corresponding to fmax is returned,
c and jflg is set    
      include 'hcscom13_2.i'

c local variables      

      real rn(0:maxage),fact,delta,yy,y1,y2
      integer jflg,isel
      jflg=0
      if (yy.lt.0.0001) then
      fact=0.0
      goto 99
      endif
c if the catch cannot be obtained by F=fmax,
c set jflg, return the catch corresponding to fmax
c and leave    
      call yield(rn,y1,fmax,isel)
      if (y1.lt.yy) then
      jflg=1
      fact=fmax
      goto 99
      endif
c Start searching
      delta=0.5
      jflg=0
      fact=0.5
      call yield(rn,y1,fact,isel)
      
10    if (y1.gt.yy) then
c current F too high (y1>yy)
       if (fact.lt.0.001) then
        fact=0.0
        yy=0.0
        jflg=2
        goto 99
       endif
       fact=fact*(1.0-delta)
      else

       fact=fact*(1.0+delta)
      endif

      y2=y1
      call yield(rn,y1,fact,isel)
      if ((y1-yy)*(y2-yy).le.0.0) then
c reduce delta if straddling over the target
       delta=delta*0.6
       if (delta.lt.0.00001) goto 99
      endif
      goto 10
      
99    if (jflg.gt.0) call yield(rn,yy,fact,isel)
c      write(*,*) 'findf',fact
      end
 

      subroutine yield(rn,y,fact,isel)
c computes the yield (= y)from stock numbers and f-factor (=ff)  
c and the current weights  
      include 'hcscom13_2.i'
c local variables
      real fact,y,rn(0:maxage)  
      integer isel   
      y=0.0
      do 22 ia=0,iaold
      z=rm(ia)+sel(ia,isel)*fact
      if (z.gt.0.0) then
      y=y+rn(ia)*sel(ia,isel)*fact*(1.0-exp(-z))/z*weca(ia)   
      else
      y=0.0
      endif
22    continue     
      end

      subroutine yielda(rn,y,cn,fact,agecatch,isel)
c computes the yield (= y)from stock numbers and f-factor (=ff)  
c and also the mean age in the catch, for use in the yieldrecr subroutine  
      include 'hcscom13_2.i'
c local variables
      real fact,y,rn(0:maxage),agecatch,cn(0:maxage)  
      integer isel   
      y=0.0
      ay=0.0
      do 22 ia=0,iaold
      z=rm(ia)+sel(ia,isel)*fact
      if (z.gt.0.0) then
      cn(ia)=rn(ia)*sel(ia,isel)*fact*(1.0-exp(-z))/z
      y=y+cn(ia)*weca(ia) 
      ay=ay+float(ia)*cn(ia)*weca(ia)   
      else
      y=0.0
      cn(ia)=0.0
      endif
22    continue
      if (y.gt.0.0) then
      agecatch=ay/y     
      else
      agecatch=-1.0
      endif
      end

      
      subroutine calcssb(rn,s,fact,isel)
      include 'hcscom13_2.i'
c local variables
      real rn(0:maxage),s
      integer isel
      s=0.0
      do 22 ia=0,iaold
      s=s+rn(ia)*exp(-sel(ia,isel)*fact*propf-rm(ia)*propm)
     &*west(ia)*rmat(ia)  
c      write(*,*) 'calcssb',ia,isel,fact,rm(ia),sel(ia,isel),west(ia),
c     &rmat(ia),s
22    continue     
      end

      subroutine calcssba(rn,s,sn,fact,agessb,isel)
c calculates the mean age at spawning in addition to the SSB
c For use in the yield/recruit routine
c Timing is dedtermined by propm
      include 'hcscom13_2.i'
c local variables
      real rn(0:maxage),sn(0:maxage),s,fact,agessb
      integer isel
      s=0.0
      sa=0.0
      do 22 ia=0,iaold
      sn(ia)=rn(ia)*exp(-sel(ia,isel)*fact*propf-rm(ia)*propm)*rmat(ia)
      s=s+sn(ia)*west(ia)
      sa=sa+float(ia)*sn(ia)*west(ia)  
22    continue 
      agessb=sa/s    
      end      

      subroutine calctsb(rn,tsb)
      include 'hcscom13_2.i'
      real rn(0:maxage),tsb
      tsb=0.0
      do 10 ia=itsbage,iaold
      tsb=tsb+rn(ia)*west(ia)
10    continue
      end    
 
      subroutine calcbplus(rn,bs)
c Note: The B+ biomass in the HR rule uses catch weights!
      include 'hcscom13_2.i'
      real rn(0:maxage),bs
      bs=0.0
      do 10 ia=ibplusage,iaold
      bs=bs+rn(ia)*weca(ia)
10    continue
      end      

      subroutine updatewema(iy,rn)
c updates weights and maturities for next year,
c Uses weights from year before to calculate tsb
c called before the update of the stocknumbers in nextyear
      include 'hcscom13_2.i'
      real rn(0:maxage)
      integer iy

      call calctsb(rn,tsb)      
c Periodic trend - year factor
      pfactor=1.0+wampl*cos(6.283185*(float(iy)-wphase)/
     &wperiod)  
c random factor for the year
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*wsigma 
      randfactory=xm 
c Age loop
      do ia=lowage,iaold
c year class factor
      if (iy.ge.-iaold) then
      yclfct=wyclfct(iy-ia)
      else
      yclfct=1.0
      endif
      call calcwema(ia,tsb,pfactor,randfactory,yclfct,1)
      trueweca(iy+1,ia)=weca(ia)
      truewest(iy+1,ia)=west(ia)
      truemat(iy+1,ia)=rmat(ia)
      enddo
      end
 
      subroutine calcwema
     &(ia,tsb,pfactor,randfactory,yclfct,kflg)
      include 'hcscom13_2.i'
      integer ia,kflg
c kflg=0 means that no age-dependent random term (for use in Yieldrecr)
c else 1
      real tsb,pfactor,randfactory,yclfct
c Density dependence and mulitpliers (year class factor is generated in recruit)
c when the year class is born
      if (kflg.ne.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*westspec(ia,5)
      randfactora=xm 
      else
      randfactora=1.0
      endif
      west(ia) = westspec(ia,1)*(1.0+westspec(ia,4)*(wbref-tsb)/wbref)*
     &pfactor*randfactory*randfactora*yclfct
      if (kflg.ne.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*wecaspec(ia,5)
      randfactora=xm 
      else
      randfactora=1.0
      endif
      weca(ia) = wecaspec(ia,1)*(1.0+wecaspec(ia,4)*(wbref-tsb)/wbref)*
     &pfactor*randfactory*randfactora*yclfct
c Density dependence of maturation is by referring to an age agemat that 
c is different from the actual age
      if (kflg.ne.0) then
      x=rnx(rintvar)
      xsi=snrn(x)
      xm=1.0+xsi*voksenspec(ia,5)
      randfactora=xm 
      else
      randfactora=1.0
      endif
      agemat = float(ia)*(1.0+voksenspec(ia,4)*(wbref-tsb)/wbref)*
     &pfactor*randfactory*randfactora*yclfct
c Find maturation corresponding to age = agemat
      i1=int(agemat)
      i2=i1+1
      if (i1.lt.lowage) then
      rmat(ia)= voksenspec(lowage,1)
      else if (i2.gt.iaold) then
      rmat(ia)=voksenspec(iaold,1)
      else
      rmat(ia)=voksenspec(i1,1)+(voksenspec(i2,1)-voksenspec(i1,1))*
     &(agemat-float(i1))
      endif
 
c Outer bounds
      if (weca(ia).lt.wecaspec(ia,2)) weca(ia)=wecaspec(ia,2)
      if (weca(ia).gt.wecaspec(ia,3)) weca(ia)=wecaspec(ia,3)
      if (west(ia).lt.westspec(ia,2)) west(ia)=westspec(ia,2)
      if (west(ia).gt.westspec(ia,3)) west(ia)=westspec(ia,3)
      if (rmat(ia).lt.voksenspec(ia,2)) rmat(ia)=voksenspec(ia,2)
      if (rmat(ia).gt.voksenspec(ia,3)) rmat(ia)=voksenspec(ia,3)
c extra security on rmat
      if (rmat(ia).gt.1.0) rmat(ia)=1.0
      if (rmat(ia).lt.0.0) rmat(ia)=0.0

      end




      
      
      subroutine nextyear(rn,fact,s,ps,isel)
c Project the stock forward for one year, according to
c a fishing mortality fact.
c Draw and include a new year class
c The stock is represented in rn, which is updated
      include 'hcscom13_2.i'
c local variables
      real rn(0:maxage),fact,s,ps
      integer isel
      do 20 ia=iaold-2,0,-1
      rn(ia+1)=rn(ia)*exp(-rm(ia)-sel(ia,isel)*fact)
20    continue
c plus gp
      rn(iaold)=rn(iaold)*exp(-rm(iaold)-sel(iaold,isel)*fact)+ 
     &rn(iaold-1)*exp(-rm(iaold-1)-sel(iaold-1,isel)*fact)  
c recruits according to the s in the call
      call recr(r0,s,ps)
      rn(0)=r0
      end
      
      subroutine rcrparam(iy,jflag) 
c Year changes in recruitment parameter a.
c Called before nextyear, i.e. when projecting the true stock,
c or when needing a fresh recruitment estimate
c asrparam and bsrparam are seen by the recruitment routine
c They are always reset to the originals when leaving the recruitment
c routine
c jflag = 0 signals that the originals shall be used. That ensures that 
c the current asrparam allways has a value if this routine is called before 
c recruitment is invoked
      include 'hcscom13_2.i'
c local variables
      integer iy,jflag
c transfer current recruitment parameters to working variables
      ireg=nregact(iy)
      asrorg=recrparam(ireg,1)   
      bsrparam=recrparam(ireg,2)   
      sigmar=recrparam(ireg,3)   
      rtrunc1=recrparam(ireg,4)   
      rtrunc2=recrparam(ireg,5)   
      depens=recrparam(ireg,6)   
      ramplitude=recrparam(ireg,7)   
      rperiod=recrparam(ireg,8)   
      rpahse=recrparam(ireg,9)   
      rinterval=recrparam(ireg,10)   
      rintvcv=recrparam(ireg,11)   
      spasmfct=recrparam(ireg,12)   
      jfunc=irecrcode(ireg,1)
      jdist=irecrcode(ireg,2)
      jtrunc=irecrcode(ireg,3)
c periodic variation
      if (jflag.ge.1) then
      asrparam1=asrorg*(1.0+ramplitude*
     &cos(6.28*(float(iy)-rphase)/rperiod))
c spasmodic kicks
      if (iy.le.maxyr.and.iy.ge.0) then
      if (ispasm(iy).eq.1) then
      asrparam=asrparam1*spasmfct
      else
      asrparam=asrparam1
      endif
      endif
      else
      asrparam=asrorg
c alternative: Use geomean rnobs
c could be written
      endif
      end
      

 
      subroutine recr(r0,s,ps)
c generates a recruitment r0 according to the the specified function with
c SSB = function of s (transfer ->s))*noise multiplier
c Parameters:
c a-parameter, called asrparam (common)
c b-parameter, called bsrparam (common) 
c Variance (transfer ->ps) This is sigma in a normal distribution
c truncation limit called rtrunc (common)
c - to ensure standard recruitment, set s > breakssb in the call
c
c Functions (jfunc: common) - generates deterministic values r1
c 1: Hockey stick r1=a if s>b; else r=a*s/b
c 2: Beverton - Holt r1=a*s/(b+s)
c 3: Ricker r1=a*s*exp(-b*s)

c Distributions of noise (jdist:common)
c 1. Normal distribution 
c 2. Lognormal distribution

c Noise is implemented through a multiplier xm derived from xsi
c xsi is a random number with standard normal distribution
c Normal distribution: xm = (1+ps*xsi): 
c Lognormal distribution: xm = exp(ps*xsi)
c The final recruitment is  r = r1*xm

c Truncation (jtrunc: common): 
c 1: xsi > trunc or xsi < -rtrunc: 
c 2: xm > trunc or xm < 1/trunc
c Here, xsi is the standard normal random number.
c xm is the final multiplier:
c  For normal distribution: xm = 1+ps*xsi
c  For lognormal distribution: xm = exp(ps*xsi)
c
c For lognormal distributions, options 1 and 2 are equivalent
c For normal distributions, both are relevant: 
c 1 gives symmetry in abs. terms, 2 in relative terms

c The a-parameter can change from year to year

      include 'hcscom13_2.i'
c local variables
      real r0,r1,s,ps

c find mean recruitment r1
      if (jfunc.eq.1) then
       if (s.lt.bsrparam) then     
       r1=asrparam*(s/bsrparam)**depens
       else
        r1=asrparam
       endif 
      else if (jfunc.eq.2) then
       r1= asrparam*s**depens/(s**depens+bsrparam**depens)
c       write(*,*) 'recr',asrparam,s,bsrparam,depens,r1,ps          
      else if (jfunc.eq.3) then
       r1=asrparam*s/bsrparam*exp(1.0-s/bsrparam)
      else
       write(*,*) 'Incomprehensible recruit function', jfunc
      endif
      if (ps.gt.0.0) then
c case stochastic recruitment      
c draw random number, convert to norm multiplier
10    x=rnx(rintvar)
      xsi=snrn(x)
c convert to multiplier xm      

      if (jdist.eq.1) then
c Normal error, ps is CV 
      xm=1.0+xsi*ps 
      else if (jdist.eq.2) then
      xm=exp(ps*xsi-ps**2/2.0)
      else
      write(*,*) 'Incomprehensible recruit distribution', jdist
      stop
      endif
c Truncation
c Variant 1: Trunc is lower and upper number of SDs (lower negative).
      if (jtrunc.eq.1) then
       if (xsi.lt.rtrunc1.or.xsi.gt.rtrunc2) then
        goto 10
        endif
      else
c Variant 2:  upper and lower bounds on the multiplier,
       if (xm.lt.rtrunc1.or.xm.gt.rtrunc2) then
        goto 10
       endif
      endif
c Final nummber
      r0=xm*r1  
C Final check, don't accept a negative number
      if (r0.lt.0.0) goto 10      
      else
c case deterministic recruitment
      r0=r1
      endif
c      write(*,*) 'in recr',asrparam,ps,r0,xm

c calculate the year class factor for weights and maturities
c This can always be calculated, but is only to be used 
c when recruit is used to make a new real year class
      wyclfct0=1.0+wyclalfa*(wrref-r0)/wrref
      if (wyclfct0.lt.wycllow) wyclfct0=wycllow
      if (wyclfct0.gt.wyclhigh) wyclfct0=wyclhigh
c reset of the recruitment parameters to originals, to avoid
c getting stuck with wrong values in the wrong places
      asrparam=asrorg
      bsrparam=bsrorg
      end

      subroutine yieldrecr
c Stand-alone routine to calculate yield and SSB per recruit
c at equilibrium between F=0 and Fcrash
c If there is spasmodic recruitment, it is assumed that the a-parameter
c reflects the basic recruitment. The recruitment level
c corresponds to the average recruitment, taking the spasmodic
c multiplier into account.
c
c If there is more than one recruitment regime, the routine here is repeated
c for each regime.
      include 'hcscom13_2.i'
      real rn(0:maxage),cn(0:maxage),sn(0:maxage)
c Set up population properties and initialise arrays
      do ireg=1,nregimes
c transfer current recruitment parameters to working variables
      asrorg=recrparam(ireg,1)   
      asrparam=recrparam(ireg,1)   
      bsrparam=recrparam(ireg,2)   
      sigmar=recrparam(ireg,3)   
      rtrunc1=recrparam(ireg,4)   
      rtrunc2=recrparam(ireg,5)   
      depens=recrparam(ireg,6)   
      ramplitude=recrparam(ireg,7)   
      rperiod=recrparam(ireg,8)   
      rpahse=recrparam(ireg,9)   
      rinterval=recrparam(ireg,10)   
      rintvcv=recrparam(ireg,11)   
      spasmfct=recrparam(ireg,12)   
      jfunc=irecrcode(ireg,1)
      jdist=irecrcode(ireg,2)
      jtrunc=irecrcode(ireg,3)
      write (*,*) '********************************'
      write (*,*) 'Y/R for recruitment regime: ',ireg
      write (*,*) '********************************'
      if (ioutopt(22).eq.1)  then

      write (22,*) '********************************'
      write (22,*) 'Recruitment regime: ',ireg
      write (22,*) '********************************'
      endif
      if (ioutopt(23).eq.1)  then
      write (23,*) '********************************'
      write (23,*) 'Recruitment regime: ',ireg
      write (23,*) '********************************'
      write (23,*)
      write (23,*) 'F Age 0-maxage'
      endif
      if (ioutopt(24).eq.1)  then
      write (24,*) '********************************'
      write (24,*) 'Recruitment regime: ',ireg
      write (24,*) '********************************'
      write (24,*)
      write (24,*) 'F Age 0-maxage'
      endif
c      rl=rl0
      do 10 ia=0,iaold
c print population data
      if (ioutopt(22).eq.1)write(22,*) 'age sel weca west matur',
     &ia,sel(ia,2),weca(ia),west(ia),rmat(ia)
10    continue  
     
c Prepare a-parameter etc.
      ff1=1.0
      ymax=0.0
      ffymax=0.0
      symax=0.0
      if (ioutopt(22).eq.1) write(22,*)
      if (ioutopt(22).eq.1) write(22,*) 'Yield and SSB per recruit with 
     &deterministic recruitment dependent on SSB: '
      if (spasmfct.gt.1.0) 
     &asrparam=asrorg*((spasmfct-1.0)/rinterval+1.0)
c Find multiplier for the a-parameter taking the skewed truncated distribution 
c into account   
      call truncmult(tm,ireg)
      asrparam=asrparam*tm
c Screen over a range of F-multipliers to get yield/recruit
c and other characteristics of the population
c Stop when SSB/R is close to zero (1% of a-param)
c First, find slope at the origin     
      ff=0.0001
      call eqlib(rn,s,ff,iflgd)
      call yield(rn,y,ff,1)
      slorg=y/0.0001
      ff01=-1.0
c calculate yield/recruit in an initial priming loop.
      if (ioutopt(22).eq.1) write(22,*) 'F-factor  Yield       SSB       
     &TSB   %Virg   Y/SSB Age_catch Age_SSB Recruits'
      ylast=0.0       
      nsteps=int(ff1*100.0)
      do 900 istep=0,nsteps
      ff=float(istep)*0.01
      call eqlib(rn,s,ff,iflgd)
c      write(*,*) 'call eq',ff,s
c      write(*,*)
      if (iflgd.eq.1) goto 901
      s1=s
      call calcssba(rn,s,sn,ff,agessb,1)
      call calctsb(rn,tb)
      call yielda(rn,y,cn,ff,agecatch,1)
      if (istep.eq.0) virg=s
      slope=(y-ylast)/0.01
      ylast=y
      if (ioutopt(22).eq.1) write(22,100) ff,y,s,tb,s/virg*100.0,y/s,
     &agecatch,agessb,rn(0)
      if (ioutopt(23).eq.1)write(23,*) ff,(cn(i),i=0,iaold)
      if (ioutopt(24).eq.1)write(24,*) ff,(sn(i),i=0,iaold)
100   format (f6.3,3f10.2,2f8.3,2f8.2,f10.2)      
      if (y.gt.ymax) then
      ymax=y
      ffymax=ff
      symax=s
      endif
      if (ff.gt.0.0.and.slope.lt.slorg*0.1.and.ff01.lt.0.0) then
      ff01=ff
      yy01=y
      endif
900   continue

c Find more precise F01 and Fmax
901   if (ff01.gt.0.0) then
      f01=-1.0
      ylast=yy01
      do 91 istep=0,20
      ff=ff01+float(istep-11)/1000.0
      call eqlib(rn,s,ff,iflgd)
      call calcssba(rn,s,sn,ff,agessb,1)
      call yielda(rn,y,cn,ff,agecatch,1)
      if (istep.gt.0) then
      slope=(y-ylast)/0.001
      if (ff.gt.0.0.and.slope.lt.slorg*0.1.and.f01.lt.0.0) then
      f01=ff
      s01=s
      y01=y
      endif
      endif
      ylast=y
91    continue
      else
      f01=-1.0
      y01=-1.0
      s01=-1.0
      endif

      if (ioutopt(22).eq.1)then
      write(22,111) ffymax
      write(22,115) ymax
      write(22,116) symax
      write(22,*)
      write(22,112) f01
      write(22,113) y01
      write(22,114) s01
      endif
      write(*,*)
      write(*,111) ffymax
      write(*,115) ymax
      write(*,116) symax
      write(*,*)
      write(*,112) f01
      write(*,113) y01
      write(*,114) s01
111   format('F at Ymax (approx): ',f5.2)     
115   format('Yield at Fmax: ',f8.2)
116   format('SSB at Fmax: ',f8.2)
112   format('F0.1: ',f8.3)
113   format('Yield at F0.1: ',f8.2)
114   format('SSB at F0.1: ',f8.2)
      asrparam=asrorg
c End screening
      write(*,*)
      write(*,*)
      if (ioutopt(22).eq.1)  then
      write(22,*)
      write(22,*)
      endif
      if (ioutopt(23).eq.1)  then
      write(23,*)
      write(23,*)
      endif
      if (ioutopt(24).eq.1)  then
      write(24,*)
      write(24,*)
      endif
      enddo
c end loop over regimes
      close (22)
      close (23)
      close (24)
      end

      subroutine truncmult(tm,ireg)
c Find mean multiplier caused by truncation      
      include 'hcscom13_2.i'
      real tm,prob(0:100,2)
      integer ireg
      jdist=irecrcode(ireg,2)
      jtrunc=irecrcode(ireg,3)
      sigmar=recrparam(ireg,3)   
      rtrunc1=recrparam(ireg,4)   
      rtrunc2=recrparam(ireg,5)   

c convert rtunc1 to xlow and rtrunc2 to xhigh
c Find tm = mean xsi
      if (jtrunc.eq.1) then
c truncation of std. normal random variable x    
      xlow=rtrunc1
      xhigh=rtrunc2
      else if (jtrunc.eq.2) then
c truncation of final multiplier xm: Translate the bounds to x 
      if (jdist.eq.1) then
c Normal: xm =(1+sigmar*x)      
      xlow=(rtrunc1-1.0)/sigmar
      xhigh=(rtrunc2-1.0)/sigmar
      else if (jdist.eq.2) then
c Lognormal: xm = exp(sigmar*x)      
      xlow=log(rtrunc1)/sigmar
      xhigh=log(rtrunc2)/sigmar
      endif
      endif
      rintv=(xhigh-xlow)/100.0

c scann x over interval, convert each x to xm and cumulate xsi-values and intervals 
      x=xlow+rintv/2.0
      rtp=1.0/sqrt(2.0*3.1415)
c      cumsum=0.0
      do i=1,99
c standard normal
      prob(i,1)=rtp*exp(-(x**2)/2.0)*rintv
c      cumsum=cumsum+prob(i,1)
c convert the x to xm, place the xm-values in prob(i,2)
      if (jtrunc.eq.2) then
      if (jdist.eq.1) then
      prob(i,2)= (1.0+sigmar*x)
      else if (jdist.eq.2) then
      prob(i,2)=exp(sigmar*x)
      endif
      else
      prob(i,2)=x
      endif
c ready for next step    
      x=x+rintv
      enddo
c prob(.1) now holds the probability of the middle of the interval,
c prob(.2) holds the value of xm in the middle ofthe interval     
      
c calculate the mean:
      
      summ=0.0
      do i=1,99
      summ=summ+prob(i,1)*prob(i,2)
      enddo
      tm=summ
c     tm=summ/cumsum
c      write(*,*) xlow,xhigh,tm
      end
    
      subroutine eqlib(rn,s,ff,iflgd)
c Find equilibirum SSB and R at F=ff,
c after calculating density dependent weights and maturities
c Also, provides the equilibirum N-values
      include 'hcscom13_2.i'
      real rn(0:maxage),ff,s,sperrec,rc,rnr(0:maxage)
      integer iflgd
      iflgd=0
c calculate stock numbers per recruit in rnr
c These are only dependent on the F
      rnr(0)=1.0
      do ia=0,iaold-2
      rnr(ia+1)=rnr(ia)*exp(-rm(ia)-sel(ia,1)*ff)
      enddo
      rnr(iaold)=
     &rnr(iaold-1)/(1.0-exp(-rm(iaold)-sel(iaold,1)*ff))  
c     write(*,*) ff,rnr(iaold-1),rnr(iaold)
c start with standard weights and maturities
      do ia=0,iaold
      west(ia)=westspec(ia,1)
      rmat(ia)=voksenspec(ia,1)
    
      enddo
      tsb0=-1.0
c get tsb1 according to standard weights (tsb0 negative)
      call nw(tsb0,tsb1,ff,rnr,rc,sperrec,jflg)
      tsb0=tsb1
c get tsb according to the previous tsb
      call nw(tsb0,tsb1,ff,rnr,rc,sperrec,jflg)
c check if tsb has changed.
c If not, break, the standard weights are ok.
c     write(*,*) ff,tsb0,tsb1
      if (abs(log(tsb1/tsb0)).lt.0.0001) goto 90     
c else, check if the new weights remain unchanged
      tsb0=tsb1
      call nw(tsb0,tsb1,ff,rnr,rc,sperrec,jflg)
      if (abs(log(tsb1/tsb0)).lt.0.0001) goto 90     

c if not, search
      n=0
11    call nw(tsb1,tsb2,ff,rnr,rc,sperrec,jflg)
      shift=(tsb2-tsb1)*(tsb1-tsb0)
      if (shift.gt.0.0) then
      tsb0=tsb1
      tsb1=tsb2
      goto 11
      endif
c Shift in sign, the point is somewhere between tsb0 and tsb1
      delta=(tsb1-tsb0)/10.0
      diff1=tsb1-tsb0
      tsb=tsb0-delta
20    tsb=tsb+delta
      n=n+1
      call nw(tsb,tsb1,ff,rnr,rc,sperrec,jflg)
      diff=tsb1-tsb
      if (diff*diff1.le.0.0) delta=-delta/10.0 
      diff1=diff
      if (jflg.eq.1) goto 90
c Temporary: Stop eternal loop
      if (n.gt.100) then
      write(*,*) 'no convergence',tsb0,tsb1,sperrec,rc,west(5),delta
      stop
      endif
c end temporary
      if (abs(delta).gt.0.001) goto 20
c end point when tsb is converged 
90    if (rc.lt.0.0.or.jflg.eq.1) then
      rc=0.0
      iflgd=1
      endif
      s=rc*sperrec
      do ia=0,iaold
      rn(ia)=rnr(ia)*rc
      enddo
c Now er have rn and s
      end    

      subroutine nw(tsbin,tsbout,ff,rnr,rc,sperrec,jflg)
c block of calculations belonging in sub eqlib to 
c derive a new tsbout with weights according to the input tsb
c input: tsbin, leading to weights
c rnr are stock numbers at age per recruit, do not change
c Fishing mortality just to translate rnr to Ns at spawning time
c output: new tsb
c tsb, SSB/recruit and recruitment multiplier based on the weights according to the input tsb
c weights and maturities in west and rmat are commons
      include 'hcscom13_2.i'
      real rn(0:maxage),ff,sperrec,rc,rnr(0:maxage)
      integer jflg
      jflg=0
c Set non-relevant terms to fixed values
      pfactor=1.0
      randfactor=1.0
      yclfct=1.0

c get weights from the input tsb
c Use current weights if tsbin<0
      if (tsbin.ge.0.0) then
      do ia=lowage,iaold
      call calcwema(ia,tsbin,pfactor,randfactor,yclfct,0)
      enddo 
      endif

c Get ssb/recruit with the initial weights and maturities
      sperrec=0.0
      do ia=0,iaold
      sperrec=sperrec+rnr(ia)*exp(-propm*rm(ia)-propf*sel(ia,1)*ff)
     &*west(ia)*rmat(ia)

      enddo
c derive recruitment multiplier from SR function given SSB/recr.
       if (jfunc.eq.1) then
c   hockey stick
       if (sperrec*asrparam.gt.bsrparam) then
       rc=asrparam
       else
       rc=0.0
       endif
      else if (jfunc.eq.2) then
c   Beverton-Holt
      rc=asrparam-bsrparam/sperrec
      if (rc.lt.0.0) rc=0.0
      else if (jfunc.eq.3) then
c   Ricker
      rc=(1.0-log(bsrparam/sperrec/asrparam))*bsrparam/sperrec
      endif
c     write(*,*) ff,asrparam,bsrparam,sperrec,rc
c translate to stock numbers
      if (rc.le.0.0) then

      rc=0.0
      jflg=1
      return
      endif
      do ia=0,iaold
      rn(ia)=rnr(ia)*rc
      enddo
c get tsb
      call calctsb(rn,tsbout)
      end

 
      subroutine prime(rn)
      include 'hcscom13_2.i'
c local variables
      real rn(0:maxage),canum(0:maxage)
      
c if spasmodic recruitments, ensure that the mean R in the priming is right
      if (spasmfct.gt.1.0) 
     &asrparam=asrorg*((spasmfct-1.0)/rinterval+1.0)
      rn(0)=asrparam
      do 110 ia=1,iaold
110   rn(ia)=0.0
      if (jfunc.eq.1) then
      s=bsrparam
      else if (jfunc.eq.2) then
      s=bsrparam
      else if (jfunc.eq.3) then
      s=1.0/bsrparam
      endif
      ps=0.0
c run with deterministic recruitment
c for twice iaold, to get the plus group decent as well.
      do 224 i=0,2*iaold
      ff=fprim
      call nextyear(rn,ff,s,ps,1)
224   continue

c Then, priming 2*iaold years forwards with stochastic recruitment
c according to recruitment function and weights and maturities according
c to specified variation
      ps=sigmar
      do 223 iiy=1,2*iaold
c Collect canums
      i=iiy-2*iaold
      if (i.ge.iobsstart) then
      do ia=lowage,iaold
      ftemp=ff*sel(ia,1)
      canum(ia)=rn(ia)*ftemp/(ftemp+rm(ia))*(1.0-exp(-ftemp-rm(ia)))
      enddo
      call getobscanum(canum,i)
      endif
      call calcssb(rn,s,fprim,1)
      if (iiy.ge.maxyr-lowage) then
      pastssb(2*iaold-iiy)=s
      endif
      i1=iiy-2*iaold
      call updatewema(i1,rn)
      if (spasmfct.gt.1.0) 
     &asrparam=asrorg*((spasmfct-1.0)/rinterval+1.0)

      call nextyear(rn,fprim,s,ps,1)
c pick up year class factors for weight in the primed year classes
      if (iiy.ge.iaold) wyclfct(i1)=wyclfct0
223   continue
      asrparam=asrorg
c transfer numbers also for starting year of the hcr to rntrue
      end
      
      subroutine getactn(rnout)
c get numbers from nin-file, with lognormal distribution
c based on covariance matrix in nin-file    
      include 'hcscom13_2.i'
c local variables (rnin and vcv refer to true age)
      real rnout(0:maxage),rn1(20),rn2(20),
     &cc(20,20)
      do 10 i=lowage,iaold
      rn1(i-lowage+1)=log(rnin(i))
      do 11 j=lowage,iaold
      if (idata.eq.2) then
      cc(i-lowage+1,j-lowage+1)=vcv(i,j)
      else if (idata.eq.3) then
      if (i.eq.j) then
      cc(i-lowage+1,i-lowage+1)=cvnin(i)
      else
      cc(i-lowage+1,j-lowage+1)=0.0
      endif
      endif
11    continue     

10    continue
      n=iaold-lowage+1      
c These are numbers for year 0.
      call multnorm(rn1,cc,rn2,rintvar,n)
      do 12 i=lowage,iaold
      j=i-lowage+1
c bias correction      
      rnout(i)=exp(rn2(j)-vcv(i,i)**2/2.0)
12    continue
      end
      

      subroutine bootprint 
      include 'hcscom13_2.i'      
c local variables 
      character*20 fmt713
      character*16 fmt711
      character*16 fmt712
      real tall(niter)  
c factors derived from niter 
      rniter = float(niter) 
      ip10 = int(rniter*0.1)
      ip50 = int(rniter*0.5)
      ip90 = int(rniter*0.9)
c formats derived from maxyr
      fmt711='(a10,i5,31f10.0)'      
      write(fmt711(9:10),'(i2)') maxyr+1
      fmt712='(a10,i5,31f10.3)'      
      write(fmt712(9:10),'(i2)') maxyr+1
      fmt713='(a10,i5,10x,  f10.1)'
      write(fmt713(12:13),'(i2)') maxyr
    

      
c fill in iflag: If the flag was set once, that should carry over to 
c the rest of the trajectory, We dont bother about that when calculating means etc,
c but probcrash should reflect the number of trajectories that are lost at some stage.
      do i=1,niter
      do j=1,maxyr      
      iflag(j,i)=max(iflag(j-1,i),iflag(j,i))
      enddo
      enddo
c decide the btrig- the prob of btrig is presented in several places      
      if (irule.eq.2) then
       btrig=hcrparam(2)
       else
       btrig=hcrparam(1)
       endif


c Standard table by year    
      if (ioutopt(11).eq.1) then
      write(11,*) 'Yearly true mean values'
      write(11,*) 'Year    F      SSB     Catch  TAC  AbsIAV  Ptrig   Pl
     &im   Pcrash'
      c10=0.0
      c50=0.0
      c90=0.0
      f10=0.0
      f50=0.0
      f90=0.0
      do 701 iy=0,maxyr
      fmean=0.0
      cmean=0.0
      smean=0.0
      tacmean=0.0
      varmean=0.0
      pblim=0.0
      pbtrig=0.0
      probcrash=0.0

      do 702 i=1,niter
      fmean=fmean+truef(iy,i)/rniter
      cmean=cmean+truecatch(iy,i)/rniter
      smean=smean+truessb(iy,i)/rniter
      tacmean=tacmean+tac(iy,i)/rniter
      if (iy.gt.0) then
      if(tac(iy-1,i).gt.0.0.and.tac(iy,i).gt.0.0) 
     &varmean=varmean+
     &abs((tac(iy,i)-tac(iy-1,i))/(tac(iy-1,i)+tac(iy,i))*2.0/rniter)
      endif
      if (truessb(iy,i).lt.blim) pblim=pblim+1.0/rniter
      if (decbas(iy,1,i).lt.btrig.and.iy.gt.0) pbtrig=pbtrig+1.0/rniter
      probcrash=probcrash+float(iflag(iy,i))/rniter
702   continue      
      write(11,101) iy+iy0,fmean,smean,cmean,tacmean,varmean*100.0,
     &pbtrig*100,pblim*100.0,probcrash*100.0
701   continue     
101   format (i5,2x,f7.3,3f7.0,4f7.1)    
      write(11,*)
      
c fractiles for main results
      write(11,*) 'Fractiles for TAC'
      write(11,*) 'Year        Mean       10%       50%       90%'
      do 401 iy=0,maxyr
      tacmean=0.0
      do 301 i=1,niter
      tacmean=tacmean+tac(iy,i)/rniter
      tall(i)=tac(iy,i)
301   continue
      call sort(tall)
      write(11,311) iy+iy0,tacmean,tall(ip10),tall(ip50),tall(ip90)      
311   format (i5,2x,4f10.1)
401   continue
      write(11,*)

      write(11,*) 'Fractiles for True F'
      write(11,*) 'Year    Mean    10%    50%    90%'
      do 402 iy=0,maxyr
      fmean=0.0
      do 302 i=1,niter
      fmean=fmean+truef(iy,i)/rniter
      tall(i)=truef(iy,i)
302   continue
      call sort(tall)
      write(11,312) iy+iy0,fmean,tall(ip10),tall(ip50),tall(ip90)      
312   format (i5,2x,4f7.3)
402   continue
      write(11,*)

      write(11,*) 'Fractiles for True SSB'
      write(11,*) 'Year        Mean       10%       50%       90%'
      do 423 iy=0,maxyr
      smean=0.0
      do 323 i=1,niter
      tall(i)=truessb(iy,i)
      smean=smean+truessb(iy,i)/rniter
323   continue
      call sort(tall)
      write(11,311) iy+iy0,smean,tall(ip10),tall(ip50),tall(ip90)      
423   continue
      write(11,*)      

      write(11,*) 'Fractiles for True TSB'
      write(11,*) 'Year        Mean       10%       50%       90%'
      do 403 iy=0,maxyr
      tmean=0.0
      do 303 i=1,niter
      tall(i)=truetsb(iy,i)
      tmean=tmean+truetsb(iy,i)/rniter
303   continue
      call sort(tall)
      write(11,311) iy+iy0,tmean,tall(ip10),tall(ip50),tall(ip90)      
403   continue
      write(11,*)      

      write(11,*) 'Fractiles for Recruitment'
      write(11,*) 'Year        Mean       10%       50%       90%'
      do 413 iy=0,maxyr
      rcmean=0.0
      do 313 i=1,niter
      tall(i)=truerecr(iy,i)
      rcmean=rcmean+truerecr(iy,i)/rniter
313   continue
      call sort(tall)
      write(11,311) iy+iy0,rcmean,tall(ip10),tall(ip50),tall(ip90)      
413   continue
      write(11,*)
      
      write(11,*) 'Fractiles for IAV [(TAC(y)-TAC(y-1)]/[(TAC(y-1)+TAC(y
     &))/2] in %'
      write(11,*) 'Year    Mean    10%    50%    90%'
      do 404 iy=1,maxyr
      varmean=0.0
      do 304 i=1,niter
      if (tac(iy-1,i).gt.0.0.and.tac(iy,i).gt.0.0) then
      varmean=varmean+
     &(tac(iy,i)-tac(iy-1,i))/(tac(iy-1,i)+tac(iy,i))*2.0/rniter
      tall(i)=(tac(iy,i)-tac(iy-1,i))/(tac(iy-1,i)+tac(iy,i))*200.0
      else
      tall(i)=0.0
      endif
      if (ioutopt(18).eq.1) write(18,324) i,iy,tall(i)
324   format(2i5,f7.1)      
      
304   continue
      call sort(tall)
      write(11,333) iy+iy0,varmean*100.0,
     &tall(ip10),tall(ip50),tall(ip90)
404   continue
333   format (i5,2x,4f7.1)
      write(11,*)
      endif
      
c Print years intvpr1 - intvpr2
      if (ioutopt(12).eq.1) then 
      npryears=intvpr2-intvpr1+1
      rnpryears=float(npryears)
c true catch
      do 810 i=1,niter
      tall(i)=0.0
      do 810 j=intvpr1,intvpr2
      tall(i)=tall(i)+truecatch(j,i)/rnpryears
810   continue
      call sort(tall)
      c10=tall(ip10)
      c50=tall(ip50)
      c90=tall(ip90)
c true F      
      do 811 i=1,niter
      tall(i)=0.0
      do 811 j=intvpr1,intvpr2
      tall(i)=tall(i)+truef(j,i)/rnpryears
811   continue
      call sort(tall)
      f10=tall(ip10)
      f50=tall(ip50)
      f90=tall(ip90)
c true SSB     
      do 812 i=1,niter
      tall(i)=0.0
      do 812 j=intvpr1,intvpr2
      tall(i)=tall(i)+truessb(j,i)/rnpryears
812   continue
      call sort(tall)
      s10=tall(ip10)
      s50=tall(ip50)
      s90=tall(ip90)
c true tsb
      do 863 i=1,niter
      tall(i)=0.0
      do 863 j=intvpr1,intvpr2
      tall(i)=tall(i)+truetsb(j,i)/rnpryears
863   continue
      call sort(tall)
      t10=tall(ip10)
      t50=tall(ip50)
      t90=tall(ip90)
      smean=0.0
      tmean=0.0
      fmean=0.0
      cmean=0.0
      avmean=0.0
      do 813 i=1,niter
      do 814 j=intvpr1,intvpr2
c Note: this is maximum risk in all the stated years from year intvpr1 onward
      smean=smean+truessb(j,i)/rnpryears/rniter
      tmean=tmean+truetsb(j,i)/rnpryears/rniter
      fmean=fmean+truef(j,i)/rnpryears/rniter
      cmean=cmean+truecatch(j,i)/rnpryears/rniter      
      if (tac(j-1,i).gt.0.0.and.tac(j,i).gt.0.0) then 
      riav=abs((tac(j,i)-tac(j-1,i))/(tac(j-1,i)+tac(j,i))*2.0)
      avmean=avmean+abs(riav)/rnpryears/rniter
      endif
814   continue
813   continue


c risiki
      risklim=0.0
      pbtrig=0.0
      probcrash=0.0
      do 852 k=intvpr1,intvpr2
      irflag1=0
      irflag2=0
      do 851 i=1,niter
      if (truessb(k,i).lt.blim) irflag1=irflag1+1
      if (decbas(k,1,i).lt.btrig) irflag2=irflag2+1
851   continue 
c Maximum
      if (risklim.lt.float(irflag1)/rniter) 
     &risklim=float(irflag1)/rniter
      if (pbtrig.lt.float(irflag2)/rniter) pbtrig=float(irflag2)/rniter
852   continue
      probcrash=probcrash+float(iflag(intvpr2,niter))/rniter

      if (irule.eq.1.) then     
      write(12,511) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.2) then     
      write(12,512) (hcrparam(i),i=1,6),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.3) then     
      write(12,513) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.4) then     
      write(12,514) (hcrparam(i),i=1,8),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.5) then     
      write(12,515) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.6) then     
      write(12,516) (hcrparam(i),i=1,8),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.7) then
       write(12,511) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.8) then     
      write(12,515) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      endif


      endif
  

511   format(2f10.2,3f10.3,f5.2,f10.1,f5.1,3f10.1,6f6.3,
     &4f10.1,4f8.3,12f8.1)  
512   format(3f10.2,3f10.3,f5.2,f10.2,f5.1, 2f10.2,2f10.1,2f5.2,3f6.3,
     &4f10.1,4f8.3,12f8.1) 
513   format(2f10.2,4f10.3,f5.2,f10.2,f5.1, 2f10.2,2f10.1,2f5.2,3f6.3,
     &4f10.1,4f8.3,12f8.1) 
514   format(2f10.1,4f10.3,f5.2,f10.2,f5.1, 2f10.2,2f10.1,2f5.2,3f6.3,
     &4f10.1,4f8.3,12f8.1) 
515   format(2f10.1,5f10.3,f5.2,f10.2,f5.1, 2f10.2,2f10.1,2f5.2,3f6.3,
     &4f10.1,4f8.3,12f8.1) 
516   format(3f10.1,4f10.3,f5.2,f10.2,f5.1, 2f10.2,2f10.1,2f5.2,3f6.3,
     &4f10.1,4f8.3,12f8.1) 

c Print first 5 years
      if (ioutopt(14).eq.1) then 
c First 5 years results: One line for each call
c true catch
      do 800 i=1,niter
      tall(i)=0.0
      do 800 j=1,5

      tall(i)=tall(i)+truecatch(j,i)/5.0
800   continue
      call sort(tall)
      c10=tall(ip10)
      c50=tall(ip50)
      c90=tall(ip90)
c true F      
      do 801 i=1,niter
      tall(i)=0.0
      do 801 j=1,5
      tall(i)=tall(i)+truef(j,i)/5.0
801   continue
      call sort(tall)
      f10=tall(ip10)
      f50=tall(ip50)
      f90=tall(ip90)
c true SSB     
      do 802 i=1,niter
      tall(i)=0.0
      do 802 j=1,5
      tall(i)=tall(i)+truessb(j,i)/5.0
802   continue
      call sort(tall)
      s10=tall(ip10)
      s50=tall(ip50)
      s90=tall(ip90)
c true tsb
      do 833 i=1,niter
      tall(i)=0.0
      do 833 j=1,5
      tall(i)=tall(i)+truetsb(j,i)/5.0
833   continue
      call sort(tall)
      t10=tall(ip10)
      t50=tall(ip50)
      t90=tall(ip90)
      smean=0.0
      tmean=0.0
      fmean=0.0
      cmean=0.0
      avmean=0.0

      do 803 i=1,niter
      do 804 j=1,5
c Note: this is cumulated risk in all 5 years from year 1 onward
      smean=smean+truessb(j,i)/5.0/rniter
      tmean=tmean+truetsb(j,i)/5.0/rniter
      fmean=fmean+truef(j,i)/5.0/rniter
      cmean=cmean+truecatch(j,i)/5.0/rniter      
      if (tac(j-1,i).gt.0.0.and.tac(j,i).gt.0.0) then 
      riav=abs((tac(j,i)-tac(j-1,i))/(tac(j-1,i)+tac(j,i))*2.0)
      avmean=avmean+abs(riav)/5.0/rniter
      endif
804   continue
803   continue

c risiki
      risklim=0.0
      pbtrig=0.0
      probcrash=0.0
      do 842 k=1,5
      irflag1=0
      irflag2=0
      do 841 i=1,niter
      if (truessb(k,i).lt.blim) irflag1=irflag1+1
      if (decbas(k,1,i).lt.btrig) irflag2=irflag2+1
841   continue 
      if (risklim.lt.float(irflag1)/rniter) 
     &risklim=float(irflag1)/rniter
      if (pbtrig.lt.float(irflag2)/rniter) pbtrig=float(irflag2)/rniter
842   continue
      probcrash=probcrash+float(iflag(5,niter))/rniter
      
      if (irule.eq.1.) then     
      write(14,511) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.2) then     
      write(14,512) (hcrparam(i),i=1,6),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.3) then     
      write(14,513) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.4) then     
      write(14,514) (hcrparam(i),i=1,8),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.5) then     
      write(14,515) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*1000.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.6) then     
      write(14,516) (hcrparam(i),i=1,8),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.7) then
      write(14,511) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0  
      else if (irule.eq.8) then
       write(14,515) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      endif
      endif

c File 21 is  all years 
      if (ioutopt(21).eq.1) then 
c One line for each call, as for 1-5
c true catch
      years=float(maxyr)
      do 820 i=1,niter
      tall(i)=0.0
      do 820 j=1,maxyr
      tall(i)=tall(i)+truecatch(j,i)/years
820   continue
      call sort(tall)
      c10=tall(ip10)
      c50=tall(ip50)
      c90=tall(ip90)
c true F      
      do 821 i=1,niter
      tall(i)=0.0
      do 821 j=1,maxyr
      tall(i)=tall(i)+truef(j,i)/years
821   continue
      call sort(tall)
      f10=tall(ip10)
      f50=tall(ip50)
      f90=tall(ip90)
c true SSB     
      do 822 i=1,niter
      tall(i)=0.0
      do 822 j=1,maxyr
      tall(i)=tall(i)+truessb(j,i)/years

822   continue
      call sort(tall)
      s10=tall(ip10)
      s50=tall(ip50)
      s90=tall(ip90)
      do 843 i=1,niter
      tall(i)=0.0
      do 843 j=1,maxyr
      tall(i)=tall(i)+truetsb(j,i)/years

843   continue
      call sort(tall)
      t10=tall(ip10)
      t50=tall(ip50)
      t90=tall(ip90)
      smean=0.0
      tmean=0.0
      fmean=0.0
      cmean=0.0
      avmean=0.0

      do 823 i=1,niter
      do 824 j=1,maxyr
      smean=smean+truessb(j,i)/years/rniter
      tmean=tmean+truetsb(j,i)/years/rniter
      fmean=fmean+truef(j,i)/years/rniter
      cmean=cmean+truecatch(j,i)/years/rniter      
      if (tac(j-1,i).gt.0.0.and.tac(j,i).gt.0.0) then 
      riav=abs((tac(j,i)-tac(j-1,i))/(tac(j-1,i)+tac(j,i))*2.0)
      avmean=avmean+abs(riav)/years/rniter
      endif
824   continue
823   continue


c risiki
      risklim=0.0
      pbtrig=0.0
      probcrash=0.0
      do 832 k=1,maxyr
      irflag1=0
      irflag2=0
      do 831 i=1,niter
      if (truessb(k,i).lt.blim) irflag1=irflag1+1
      if (decbas(k,1,i).lt.btrig) irflag2=irflag2+1
831   continue 
c Maximum
     
      if (risklim.lt.float(irflag1)/rniter) 
     &risklim=float(irflag1)/rniter
      if (pbtrig.lt.float(irflag2)/rniter) pbtrig=float(irflag2)/rniter
832   continue
      probcrash=probcrash+float(iflag(maxyr,niter))/rniter




      if (irule.eq.1.) then     
      write(21,511) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.2) then     
      write(21,512) (hcrparam(i),i=1,6),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.3) then     
      write(21,513) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.4) then     
      write(21,514) (hcrparam(i),i=1,8),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.5) then     
      write(21,515) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.6) then     
      write(21,516) (hcrparam(i),i=1,8),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.7) then
       write(21,511) (hcrparam(i),i=1,5),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      else if (irule.eq.8) then
      write(21,515) (hcrparam(i),i=1,7),
     &(hcrparam(i),i=11,16),(hcrparam(i),i=18,23),
     &cmean,c10,c50,c90,fmean,f10,f50,f90,smean,s10,s50,
     &s90,tmean,t10,t50,t90,avmean*100.0,risklim*100.0,
     &pbtrig*100.0,probcrash*100.0
      endif
      write(*,*) 'All years: Cmean',cmean,'SSBmean',smean,'risklim',
     &risklim*100.0

      endif


c Comparison true SSB, observed and decision SSB
c and true and observed Fs
      do 600 i=1,niter
      if (ioutopt(16).eq.1) write(16,603) i,truessb(0,i)
      do 600 iy=0,maxyr
      if (ioutopt(17).eq.1)
     &write(17,602) i,iy,obsf(iy,i),truef(iy,i)
602   format (2i5,2f7.3) 
603   format (i5,f10.0)
600   continue


c print trajectories
      if (ioutopt(15).eq.1) then
      do 710 iter=1,20
      write(15,fmt711) 'TAC       ',iter,(tac(i,iter),i=0,maxyr)
      write(15,fmt711) 'Decis_Bas ',iter,(decbas(i,1,iter),i=0,maxyr)
      write(15,fmt713) 'TACchange ',iter,
     &((tac(i,iter)-tac(i-1,iter))/tac(i-1,iter)*100.0,i=1,maxyr)
      write(15,fmt711) 'True_bas  ',iter,(truessb(i,iter),i=0,maxyr)
      write(15,fmt712) 'True_F    ',iter,(truef(i,iter),i=0,maxyr)
      write(15,fmt711) 'Recruits  ',iter,(truerecr(i,iter),i=0,maxyr)
      write(15,fmt711) 'Surpl_prd ',iter,
     &(truetsb(i+1,iter)-truetsb(i,iter)+tac(i,iter),i=0,maxyr-1)
      write(15,*)
710    continue
      endif  
      end
               
c print weights and maturities: First 10 replicas
      subroutine printwema
      include 'hcscom13_2.i'
      write(25,*) 'Iteration number',iter
      write(25,*)
      write(25,*) 'Catch weights'
      write(25,*) 'Age/Year ',(ia,ia=lowage,iaold)     
      do iy=0,maxyr
      write(25,*) '         ',iy,(trueweca(iy,ia),ia=lowage,iaold)
      enddo
      write(25,*)
      write(25,*) 'Stock weights'
      write(25,*) 'Age/Year ',(ia,ia=lowage,iaold)     
      do iy=0,maxyr
      write(25,*) '         ',iy,(truewest(iy,ia),ia=lowage,iaold)
      enddo
      write(25,*)
      write(25,*) 'Maturities'
      write(25,*) 'Age/Year  ',(ia,ia=lowage,iaold)     
      do iy=0,maxyr
      write(25,*) '         ',iy,(truemat(iy,ia),ia=lowage,iaold)
      enddo
      write(25,*)
   
      end  
      
      subroutine printsr(truen)
      include 'hcscom13_2.i'
c local variables
      real truen(0:maxage,0:maxyr)
      do 10 i=lowage+1,maxyr
10    write(20,100) iter,i,truessb(i-lowage,iter),truen(0,i)
100   format(2i5,2f10.0)
      end
      
      subroutine printbas(truen)
      include 'hcscom13_2.i'
      real truen(0:maxage,0:maxyr)
      character*17 fmt1
      fmt1='(i4,x(2x,2f10.3))'
      write(fmt1(5:5),'(i1)') nbasis
      call compbasis(truen)
      do iy=0,maxyr
      write(19,fmt1) iy,(truebas(i,iy),decbas(iy,i,iter),i=1,nbasis)
      enddo
      end
      
      subroutine printrecr
      include 'hcscom13_2.i'
      do iy=1,maxyr
      write(26,100) iter,iy,truerecr(iy,iter),obsrecr(iy,iter)
100   format (2i6,2f12.1)      
      enddo
      end
      
      
       
c Library routines
c ********************      
        function snrn(r)
c WHen r is a random number between 0 and 1
c the function returns the number with that probability
c in a standard normal distribution
      real snrn,r
      if (r.gt.0.5) then
      t=sqrt(log((1.0/(r-0.5))**2))
      sg=1.0
      else if (r.lt.0.5) then
      t=sqrt(log((1.0/(r-0.5))**2))
      sg=-1.0
      else
      sg=0.0
      endif
      snrn=sg*(t-(2.30753+0.27061*t)/
     &(1.0+0.99229*t+0.04481*t*t))
      return
      end

       
       function rnx(rintvar)
c returns a random variable between 0 and 1, adjusts the
c seed rintvar
      double precision rintvar,rndn
      real rnx
      rintvar=rintvar*23.0d0
      rintvar=dmod(rintvar,100000001.0d0)
      rndn=rintvar/100000001.0d0
      rnx=sngl(rndn)
      return
      end
      
    
      subroutine multnorm(u,cov,r,rintvar,n)
c generates a random vector of multi-normally distributed
c numbers with given mean vector and covariance matrix
c max dim is 20
      
      real covinv(20,20),u(20),r(20),cov(20,20)
      double precision rintvar
      real s1(20),u1(20)

      integer n
      r01=rnx(rintvar)
      rstn=snrn(r01)
      u1(1)=u(1)
      s1(1)=sqrt(cov(1,1))
      r(1)=u1(1)+rstn*s1(1)
      do 100 i=2,n
      r01=rnx(rintvar)
      rstn=snrn(r01)
      u1(i)=u(i)
      call syminv(cov,covinv,i)

      if (covinv(i,i).le.0.0) then
      write(*,*) 'Covariance martrix not positive definite'
      write(*,*) 'Get a better one!'
      stop
      endif
      
      s1(i)=sqrt(1.0/covinv(i,i))
      do 30 j=1,i-1
      u1(i)=u1(i)-covinv(i,j)/covinv(i,i)*(r(j)-u(j))
30    continue
      r(i)=u1(i)+rstn*s1(i)
100   continue
      end


      subroutine syminv(b,a,n)
c inverts a symmetrical matrix, n*n
c Original source: J.C.Nash,
c Compact numerical methods for computers
c Adam Hilger, Bristol 1979
c Copied from B.F.J.Manly
c Randomisation and Monte Carlo methods in biology
c Chapman & Hall London 1994
c
c The matrix B which is n*n is inverted, The result is placed in a
      real b(20,20),a(20,20),wa(210),wb(20)
c 20 er max. dim, must be adjusted as needed
c wa og wb are working matrices underway, wa must be at least
c  n*(n+1)/2. 
c
c If n=1, just turn the fraction over
      if (n.gt.1) goto 10
      if (b(1,1).eq.0.0) goto 100
      a(1,1)=1.0/b(1,1)
      return
c n>1
10    do 20 i=1,n      
      do 20 j=1,i
20    wa(i*(i-1)/2+j)=b(i,j)
      do 30 ka=1,n
      k=n+1-ka
      s=wa(1)
      m=1
      do 40 i=2,n
      iq=m
      m=m+i
      t=wa(iq+1)
      if (s.eq.0.0) goto 100
      wb(i)=-t/s
      if (i.gt.k) wb(i)=-wb(i)
      do 40 j=iq+2,m
40    wa(j-i)=wa(j)+t*wb(j-iq)
      iq=iq-1
      wa(m)=1.0/s
      do 30 i=2,n
30    wa(iq+i)=wb(i)
c result in a
      do 50 i=1,n
      do 50 j=1,i
      a(i,j)=wa(i*(i-1)/2+j)
50    a(j,i)=a(i,j)
      return
c entry if the matrix is singular
100   stop 'Singular matrix'
      end
 

      subroutine creategeo(geodist,rintv,maxyr)
      real geodist(0:maxyr),rintv
      integer maxyr
      px=0.0
      pr=1.0/rintv
      do i=1,maxyr
      flii=float(i)-1.0
      px=px+pr*exp(flii*log(1.0-pr))
      geodist(i)=px
      enddo
      end

   
      subroutine sort(x)
c sorts an array of niter elements, puts the values in
c the same array      
      include 'hcscom13_2.i'
      real x(niter),y(niter)
      y(1)=x(1)
      do 100 i=2,niter
      y(i)=0
      do 101 j=1,i
      if (x(i).lt.y(j)) then
      do 102 k=i-1,j,-1
      y(k+1)=y(k)
102   continue      
      y(j)=x(i)
      goto 100
      endif
101   continue
      y(i)=x(i)
100   continue      
      do 104 i=1,niter
      x(i)=y(i)
c     write(*,*) i,x(i),y(i)
104   continue     
      end

      subroutine regress(x,y,a,b,n,ymean)
c Estimation of parameters in a linear regression between
c x and y: y=a+bx
c n is number of obs, limited to niter here
c Etter A. Høyland: 
c Sannsynlighetsregning og statistisk metodelære, bd. 2, s. 179
c Tapir forlag, Trondheim 1973.
      include 'hcscom13_2.i'
      real x(niter),y(niter),a,b,ymean,xmean,xx,xy
      integer n
c mean x and y:
      xmean=0.0
      ymean=0.0
      do 10 i=1,n      
      xmean=xmean+x(i)
      ymean=ymean+y(i)
10    continue
      xmean=xmean/float(n)
      ymean=ymean/float(n)
c     sums xy and xx
      xy=0.0
      xx=0.0
      do 11 i=1,n
      xx=xx+(x(i)-xmean)**2
      xy=xy+(x(i)-xmean)*y(i)
11    continue
      b=xy/xx
      a=ymean-b*xmean
      end
